@@ -17,7 +17,6 @@
  */
 #include "standard_actions_module.h"
 
-#include <QLayout>
 
 #include <KAction>
 #include <KActionCollection>
@@ -29,6 +28,8 @@
 #include <KMessageBox>
 #include <KLocale>
 
+#include <QVBoxLayout>
+
 K_PLUGIN_FACTORY(StandardActionsModuleFactory, registerPlugin<StandardActionsModule>();)
 K_EXPORT_PLUGIN(StandardActionsModuleFactory("kcm_standard_actions"))
 
@@ -41,7 +42,7 @@ StandardActionsModule::StandardActionsModule(
       ,m_actionCollection(NULL)
     {
     // Configure the KCM
-    KCModule::setButtons(KCModule::Buttons(KCModule::Default | KCModule::Apply));
+    KCModule::setButtons(KCModule::Buttons(KCModule::Default | KCModule::Apply | KCModule::Help));
 
     // Create and configure the editor
     m_editor = new KShortcutsEditor(this, KShortcutsEditor::AllActions);
