@@ -29,47 +29,59 @@
 
 namespace KHotKeys {
 
+ShortcutTriggerVisitor::~ShortcutTriggerVisitor()
+    {}
+
+
 ShortcutTrigger::ShortcutTrigger(
         ActionData* data_P,
         const KShortcut& shortcut,
         const QUuid &uuid )
-    : Trigger( data_P ), _uuid(uuid), _shortcut(shortcut)
+    :   Trigger( data_P ),
+        _uuid(uuid),
+        _active(false),
+        _shortcut(shortcut)
     {
     }
 
 
-ShortcutTrigger::ShortcutTrigger(
-        KConfigGroup& cfg_P
-       ,ActionData* data_P )
-    :   Trigger( cfg_P, data_P ),
-        _uuid( cfg_P.readEntry( "Uuid", QUuid::createUuid().toString())),
-        _shortcut()
-    {
-    QString shortcutString = cfg_P.readEntry( "Key" );
-
-    // TODO: Check if this is still necessary
-    shortcutString.replace("Win+", "Meta+"); // Qt4 doesn't parse Win+, avoid a shortcut without modifier
-
-    _shortcut.setPrimary(shortcutString);
-    }
-
-
 ShortcutTrigger::~ShortcutTrigger()
     {
     keyboard_handler->removeAction( _uuid );
     }
 
 
+void ShortcutTrigger::accept(TriggerVisitor& visitor)
+    {
+    if (ShortcutTriggerVisitor *v = dynamic_cast<ShortcutTriggerVisitor*>(&visitor))
+        {
+        v->visit(*this);
+        }
+    else
+        {
+        kDebug() << "Visitor error";
+        }
+    }
+
+
 void ShortcutTrigger::aboutToBeErased()
     {
     disable();
     }
 
 
-void ShortcutTrigger::activate( bool activate_P )
+void ShortcutTrigger::activate( bool newState )
     {
-    kDebug() << activate_P;
-    if(activate_P)
+#ifdef KHOTKEYS_TRACE
+    kDebug() << "new:" << newState << "old:" << _active;
+#endif
+    // If there is no change in state just return.
+    if (newState == _active)
+        return;
+
+    _active = newState;
+
+    if (_active)
         {
         QString name = data
             ? data->name()
@@ -90,8 +102,7 @@ void ShortcutTrigger::activate( bool activate_P )
         connect(
             act, SIGNAL(globalShortcutChanged(const QKeySequence&)),
             this, SIGNAL(globalShortcutChanged(const QKeySequence&)));
-            _conditions_met = true;
-            }
+        }
     else
         {
         // Disable the trigger. Delete the action.
@@ -129,12 +140,16 @@ const QString ShortcutTrigger::description() const
 
 void ShortcutTrigger::disable()
     {
+    activate(false);
+
     // Unregister the shortcut with kglobalaccel
     KAction *action = qobject_cast<KAction*>(keyboard_handler->getAction( _uuid ));
     if(action)
         {
         // In case the shortcut was changed from the kcm.
         _shortcut = action->globalShortcut();
+
+
         // Unregister the global shortcut.
         action->forgetGlobalShortcut();
         keyboard_handler->removeAction(_uuid);
