@@ -46,6 +46,52 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 namespace KWin
 {
 
+//---------------------
+// Static
+
+static QByteArray readWindowProperty( Window win, long atom, long type, int format )
+    {
+    int len = 32768;
+    for(;;)
+        {
+        unsigned char* data;
+        Atom rtype;
+        int rformat;
+        unsigned long nitems, after;
+        if( XGetWindowProperty( QX11Info::display(), win,
+            atom, 0, len, False, AnyPropertyType,
+            &rtype, &rformat, &nitems, &after, &data ) == Success )
+            {
+            if( after > 0 )
+                {
+                XFree( data );
+                len *= 2;
+                continue;
+                }
+            if( long( rtype ) == type && rformat == format )
+                {
+                int bytelen = format == 8 ? nitems : format == 16 ? nitems * sizeof( short ) : nitems * sizeof( long );
+                QByteArray ret( reinterpret_cast< const char* >( data ), bytelen );
+                XFree( data );
+                return ret;
+                }
+            else // wrong format, type or something
+                {
+                XFree( data );
+                return QByteArray();
+                }
+            }
+        else // XGetWindowProperty() failed
+            return QByteArray();
+        }
+    }
+
+static void deleteWindowProperty( Window win, long int atom )
+    {
+    XDeleteProperty( QX11Info::display(), win, atom );
+    }
+
+//---------------------
 
 EffectsHandlerImpl::EffectsHandlerImpl(CompositingType type)
     : EffectsHandler(type)
@@ -167,6 +213,14 @@ void EffectsHandlerImpl::postPaintWindow( EffectWindow* w )
     // no special final code
     }
 
+bool EffectsHandlerImpl::providesResizeEffect()
+    {
+    for( int i = 0; i < loaded_effects.size(); ++i )
+        if( loaded_effects.at(i).second->isResizeEffect() )
+            return true;
+    return false;
+    }
+
 void EffectsHandlerImpl::drawWindow( EffectWindow* w, int mask, QRegion region, WindowPaintData& data )
     {
     if( current_draw_window < loaded_effects.size())
@@ -192,6 +246,11 @@ bool EffectsHandlerImpl::hasDecorationShadows() const
     return Workspace::self()->hasDecorationShadows();
     }
 
+bool EffectsHandlerImpl::decorationsHaveAlpha() const
+    {
+    return Workspace::self()->decorationHasAlpha();
+    }
+
 // start another painting pass
 void EffectsHandlerImpl::startPaint()
     {
@@ -417,6 +476,12 @@ void EffectsHandlerImpl::propertyNotify( EffectWindow* c, long atom )
         ep.second->propertyNotify( c, atom );
     }
 
+void EffectsHandlerImpl::numberDesktopsChanged( int old )
+    {
+    foreach( const EffectPair &ep, loaded_effects )
+        ep.second->numberDesktopsChanged( old );
+    }
+
 void EffectsHandlerImpl::registerPropertyType( long atom, bool reg )
     {
     if( reg )
@@ -428,6 +493,16 @@ void EffectsHandlerImpl::registerPropertyType( long atom, bool reg )
         }
     }
 
+QByteArray EffectsHandlerImpl::readRootProperty( long atom, long type, int format ) const
+    {
+    return readWindowProperty( rootWindow(), atom, type, format );
+    }
+
+void EffectsHandlerImpl::deleteRootProperty( long atom ) const
+    {
+    deleteWindowProperty( rootWindow(), atom );
+    }
+
 void EffectsHandlerImpl::activateWindow( EffectWindow* c )
     {
     if( Client* cl = dynamic_cast< Client* >( static_cast<EffectWindowImpl*>(c)->window()))
@@ -486,6 +561,11 @@ void EffectsHandlerImpl::setCurrentDesktop( int desktop )
     Workspace::self()->setCurrentDesktop( desktop );
     }
 
+void EffectsHandlerImpl::setNumberOfDesktops( int desktops )
+    {
+    Workspace::self()->setNumberOfDesktops( desktops );
+    }
+
 QSize EffectsHandlerImpl::desktopGridSize() const
     {
     return Workspace::self()->desktopGridSize();
@@ -1279,48 +1359,21 @@ QRect EffectWindowImpl::contentsRect() const
     return QRect( toplevel->clientPos(), toplevel->clientSize());
     }
 
+QRect EffectWindowImpl::decorationInnerRect() const
+    {
+    Client *client = dynamic_cast<Client*>(toplevel);
+    return client ? client->transparentRect() : contentsRect();
+    }
+
 QByteArray EffectWindowImpl::readProperty( long atom, long type, int format ) const
     {
-    int len = 32768;
-    for(;;)
-        {
-        unsigned char* data;
-        Atom rtype;
-        int rformat;
-        unsigned long nitems, after;
-        if( XGetWindowProperty( QX11Info::display(), window()->window(),
-            atom, 0, len, False, AnyPropertyType,
-            &rtype, &rformat, &nitems, &after, &data ) == Success )
-            {
-            if( after > 0 )
-                {
-                XFree( data );
-                len *= 2;
-                continue;
-                }
-            if( long( rtype ) == type && rformat == format )
-                {
-                int bytelen = format == 8 ? nitems : format == 16 ? nitems * sizeof( short ) : nitems * sizeof( long );
-                QByteArray ret( reinterpret_cast< const char* >( data ), bytelen );
-                XFree( data );
-                return ret;
-                }
-            else // wrong format, type or something
-                {
-                XFree( data );
-                return QByteArray();
-                }
-            }
-        else // XGetWindowProperty() failed
-            return QByteArray();
-        }
+    return readWindowProperty( window()->window(), atom, type, format );
     }
 
 void EffectWindowImpl::deleteProperty(long int atom) const
-{
-    XDeleteProperty( QX11Info::display(), window()->window(), atom );
-}
-
+    {
+    deleteWindowProperty( window()->window(), atom );
+    }
 
 bool EffectWindowImpl::isMovable() const
     {
@@ -1491,6 +1544,13 @@ EffectWindowList EffectWindowImpl::mainWindows() const
     return EffectWindowList();
     }
 
+bool EffectWindowImpl::isSkipSwitcher() const
+    {
+    if( Client* c = dynamic_cast< Client* >( toplevel ))
+        return c->skipSwitcher();
+    return false;
+    }
+
 WindowQuadList EffectWindowImpl::buildQuads( bool force ) const
     {
     return sceneWindow()->buildQuads( force );
