@@ -46,6 +46,13 @@ BoxSwitchEffect::BoxSwitchEffect()
     , painting_desktop( 0 )
     , animation( false )
     , highlight_is_set( false )
+    , primaryTabBox( true )
+    , secondaryTabBox( false )
+    , mProxy( this )
+    , mProxyActivated( 0 )
+    , mProxyAnimateSwitch( 0 )
+    , mProxyShowText( 0 )
+    , mPositioningFactor( 0.5f )
     {
     text_font.setBold( true );
     text_font.setPointSize( 12 );
@@ -75,17 +82,22 @@ void BoxSwitchEffect::reconfigure( ReconfigureFlags )
     bg_opacity = conf.readEntry( "BackgroundOpacity", 25 ) / 100.0;
     elevate_window = conf.readEntry( "ElevateSelected", true );
     mAnimateSwitch = conf.readEntry( "AnimateSwitch", false );
+
+    primaryTabBox = conf.readEntry( "TabBox", true );
+    secondaryTabBox = conf.readEntry( "TabBoxAlternative", false );
     }
 
 void BoxSwitchEffect::prePaintWindow( EffectWindow* w, WindowPrePaintData& data, int time )
     {
-    if( activeTimeLine.value() != 0.0 )
+    if( activeTimeLine.value() != 0.0 && !mProxyActivated )
         {
-        if( mMode == TabBoxWindowsMode )
+        if( mMode == TabBoxWindowsMode || mMode == TabBoxWindowsAlternativeMode )
             {
             if( windows.contains( w ))
                 {
-                if( w != selected_window )
+                if( w == selected_window )
+                    w->enablePainting( EffectWindow::PAINT_DISABLED_BY_CLIENT_GROUP );
+                else
                     data.setTranslucent();
                 w->enablePainting( EffectWindow::PAINT_DISABLED_BY_MINIMIZE | EffectWindow::PAINT_DISABLED_BY_DESKTOP );
                 }
@@ -128,36 +140,11 @@ void BoxSwitchEffect::prePaintScreen( ScreenPrePaintData& data, int time )
 void BoxSwitchEffect::paintScreen( int mask, QRegion region, ScreenPaintData& data )
     {
     effects->paintScreen( mask, region, data );
-    if( mActivated )
+    if( mActivated && !mProxyActivated )
         {
-        if( mMode == TabBoxWindowsMode )
+        if( mMode == TabBoxWindowsMode || mMode == TabBoxWindowsAlternativeMode )
             {
-            thumbnailFrame.render( region );
-
-            if( mAnimateSwitch )
-                {
-                // HACK: PaintClipper is used because window split is somehow wrong if the height is greater than width
-                PaintClipper::push( frame_area );
-                paintHighlight( highlight_area );
-                foreach( EffectWindow* w, windows.keys())
-                    {
-                    paintWindowThumbnail( w );
-                    paintWindowIcon( w );
-                    }
-                PaintClipper::pop( frame_area );
-                }
-            else
-                {
-                foreach( EffectWindow* w, windows.keys())
-                    {
-                    if( w == selected_window )
-                        {
-                        paintHighlight( windows[ w ]->area );
-                        }
-                    paintWindowThumbnail( w );
-                    paintWindowIcon( w );
-                    }
-                }
+            paintWindowsBox(region);
             }
         else
             {
@@ -165,12 +152,12 @@ void BoxSwitchEffect::paintScreen( int mask, QRegion region, ScreenPaintData& da
                 {
                 thumbnailFrame.render( region );
 
-                foreach( painting_desktop, desktops.keys())
+                QHash< int, ItemInfo* >::const_iterator i;
+                for( i = desktops.constBegin(); i != desktops.constEnd(); ++i )
                     {
+                    painting_desktop = i.key();
                     if( painting_desktop == selected_desktop )
-                        {
                         paintHighlight( desktops[ painting_desktop ]->area ); //effects->desktopName( painting_desktop )
-                        }
 
                     paintDesktopThumbnail( painting_desktop );
                     }
@@ -180,6 +167,36 @@ void BoxSwitchEffect::paintScreen( int mask, QRegion region, ScreenPaintData& da
         }
     }
 
+void BoxSwitchEffect::paintWindowsBox(const QRegion& region)
+    {
+    thumbnailFrame.render( region );
+
+    if( (mAnimateSwitch && !mProxyActivated) || (mProxyActivated && mProxyAnimateSwitch) )
+        {
+        // HACK: PaintClipper is used because window split is somehow wrong if the height is greater than width
+        PaintClipper::push( frame_area );
+        paintHighlight( highlight_area );
+        QHash< EffectWindow*, ItemInfo* >::const_iterator i;
+        for( i = windows.constBegin(); i != windows.constEnd(); ++i )
+            {
+            paintWindowThumbnail( i.key() );
+            paintWindowIcon( i.key() );
+            }
+        PaintClipper::pop( frame_area );
+        }
+    else
+        {
+        QHash< EffectWindow*, ItemInfo* >::const_iterator i;
+        for( i = windows.constBegin(); i != windows.constEnd(); ++i )
+            {
+            if( i.key() == selected_window )
+                paintHighlight( i.value()->area );
+            paintWindowThumbnail( i.key() );
+            paintWindowIcon( i.key() );
+            }
+        }
+    }
+
 void BoxSwitchEffect::postPaintScreen()
     {
     if( mActivated && activeTimeLine.value() != 1.0 )
@@ -209,7 +226,8 @@ void BoxSwitchEffect::postPaintScreen()
 
 void BoxSwitchEffect::paintWindow( EffectWindow* w, int mask, QRegion region, WindowPaintData& data )
     {
-    if(( mActivated && mMode == TabBoxWindowsMode ) || ( !mActivated && activeTimeLine.value() != 0.0 ))
+    if((( mActivated && (mMode == TabBoxWindowsMode || mMode == TabBoxWindowsAlternativeMode) )
+        || ( !mActivated && activeTimeLine.value() != 0.0 )) && !mProxyActivated)
         {
         if( windows.contains( w ) && w != selected_window )
             {
@@ -226,19 +244,22 @@ void BoxSwitchEffect::paintWindow( EffectWindow* w, int mask, QRegion region, Wi
 void BoxSwitchEffect::windowInputMouseEvent( Window w, QEvent* e )
     {
     assert( w == mInput );
+    Q_UNUSED( w );
     if( e->type() != QEvent::MouseButtonPress )
         return;
     QPoint pos = static_cast< QMouseEvent* >( e )->pos();
     pos += frame_area.topLeft();
 
     // determine which item was clicked
-    if( mMode == TabBoxWindowsMode )
+    if( mMode == TabBoxWindowsMode || mMode == TabBoxWindowsAlternativeMode )
         {
-        foreach( EffectWindow* w, windows.keys())
+        QHash< EffectWindow*, ItemInfo* >::const_iterator i;
+        for( i = windows.constBegin(); i != windows.constEnd(); ++i )
             {
-            if( windows[ w ]->clickable.contains( pos ))
+            if( i.value()->clickable.contains( pos ))
                 {
-                effects->setTabBoxWindow( w );
+                effects->setTabBoxWindow( i.key() );
+                break;
                 }
             }
         // special handling for second half of window in case of animation and even number of windows
@@ -254,21 +275,21 @@ void BoxSwitchEffect::windowInputMouseEvent( Window w, QEvent* e )
         }
     else
         {
-        foreach( int i, desktops.keys())
+        QHash< int, ItemInfo* >::const_iterator i;
+        for( i = desktops.constBegin(); i != desktops.constEnd(); ++i )
             {
-            if( desktops[ i ]->clickable.contains( pos ))
-                {
-                effects->setTabBoxDesktop( i );
-                }
+            if( i.value()->clickable.contains( pos ))
+                effects->setTabBoxDesktop( i.key() );
             }
         }
     }
 
 void BoxSwitchEffect::windowDamaged( EffectWindow* w, const QRect& damage )
     {
+    Q_UNUSED( damage );
     if( mActivated )
         {
-        if( mMode == TabBoxWindowsMode )
+        if( mMode == TabBoxWindowsMode || mMode == TabBoxWindowsAlternativeMode )
             {
             if( windows.contains( w ))
                 {
@@ -294,7 +315,7 @@ void BoxSwitchEffect::windowGeometryShapeChanged( EffectWindow* w, const QRect&
     {
     if( mActivated )
         {
-        if( mMode == TabBoxWindowsMode )
+        if( mMode == TabBoxWindowsMode || mMode == TabBoxWindowsAlternativeMode )
             {
             if( windows.contains( w ) && w->size() != old.size())
                 {
@@ -320,7 +341,8 @@ void BoxSwitchEffect::tabBoxAdded( int mode )
     {
     if( !mActivated )
         {
-        if( mode == TabBoxWindowsMode )
+        if( ( mode == TabBoxWindowsMode && primaryTabBox ) ||
+            ( mode == TabBoxWindowsAlternativeMode && secondaryTabBox) )
             {
             if( effects->currentTabBoxWindowList().count() > 0 )
                 {
@@ -333,7 +355,7 @@ void BoxSwitchEffect::tabBoxAdded( int mode )
                 setActive();
                 }
             }
-        else
+        else if( mode == TabBoxDesktopListMode || mode == TabBoxDesktopMode )
             { // DesktopMode
             if( effects->currentTabBoxDesktopList().count() > 0 )
                 {
@@ -356,11 +378,12 @@ void BoxSwitchEffect::tabBoxUpdated()
     {
     if( mActivated )
         {
-        if( mMode == TabBoxWindowsMode && selected_window != effects->currentTabBoxWindow() )
+        if( (mMode == TabBoxWindowsMode || mMode == TabBoxWindowsAlternativeMode)
+            && selected_window != effects->currentTabBoxWindow() )
             {
             if( selected_window != NULL )
                 {
-                if( mAnimateSwitch )
+                if( (mAnimateSwitch && !mProxyActivated) || (mProxyActivated && mProxyAnimateSwitch) )
                     {
                     int old_index = effects->currentTabBoxWindowList().indexOf( selected_window );
                     int new_index = effects->currentTabBoxWindowList().indexOf( effects->currentTabBoxWindow() );
@@ -409,12 +432,13 @@ void BoxSwitchEffect::tabBoxUpdated()
             selected_window->addRepaintFull();
             effects->addRepaint( text_area );
             }
-        else if( mMode != TabBoxWindowsMode )
+        else if( mMode != TabBoxWindowsMode && mMode != TabBoxWindowsAlternativeMode )
             { // DesktopMode
             if( desktops.contains( selected_desktop ))
                 effects->addRepaint( desktops.value( selected_desktop )->area );
             selected_desktop = effects->currentTabBoxDesktop();
-            thumbnailFrame.setText( effects->desktopName( selected_desktop ));
+            if( !mProxyActivated || mProxyShowText )
+                thumbnailFrame.setText( effects->desktopName( selected_desktop ));
             if( desktops.contains( selected_desktop ))
                 effects->addRepaint( desktops.value( selected_desktop )->area );
             effects->addRepaint( text_area );
@@ -440,7 +464,7 @@ void BoxSwitchEffect::setActive()
     qDeleteAll( windows );
     windows.clear();
 
-    if( mMode == TabBoxWindowsMode )
+    if( mMode == TabBoxWindowsMode || mMode == TabBoxWindowsAlternativeMode )
         {
         original_windows = effects->currentTabBoxWindowList();
         setSelectedWindow( effects->currentTabBoxWindow());
@@ -449,18 +473,24 @@ void BoxSwitchEffect::setActive()
         {
         original_desktops = effects->currentTabBoxDesktopList();
         selected_desktop = effects->currentTabBoxDesktop();
-        thumbnailFrame.setText( effects->desktopName( selected_desktop ));
+        if( !mProxyActivated || mProxyShowText )
+            thumbnailFrame.setText( effects->desktopName( selected_desktop ));
         }
     calculateFrameSize();
     calculateItemSizes();
-    mInput = effects->createInputWindow( this, frame_area.x(), frame_area.y(),
-        frame_area.width(), frame_area.height(), Qt::ArrowCursor );
-    effects->addRepaint( frame_area );
-    if( mMode == TabBoxWindowsMode )
+    if( !mProxyActivated )
         {
-        foreach( EffectWindow* w, windows.keys())
+        // only create the input window when effect is not activated via the proxy
+        mInput = effects->createInputWindow( this, frame_area.x(), frame_area.y(),
+                                             frame_area.width(), frame_area.height(), Qt::ArrowCursor );
+        }
+    effects->addRepaint( frame_area );
+    if( mMode == TabBoxWindowsMode || mMode == TabBoxWindowsAlternativeMode )
+        {
+        QHash< EffectWindow*, ItemInfo* >::const_iterator i;
+        for( i = windows.constBegin(); i != windows.constEnd(); ++i )
             {
-            w->addRepaintFull();
+            i.key()->addRepaintFull();
             }
         }
     }
@@ -469,17 +499,20 @@ void BoxSwitchEffect::setInactive()
     {
     mActivated = false;
     effects->unrefTabBox();
-    if( mInput != None )
+    if( !mProxyActivated && mInput != None )
         {
         effects->destroyInputWindow( mInput );
         mInput = None;
         }
-    if( mMode == TabBoxWindowsMode )
+    mProxyActivated = false;
+    mPositioningFactor = 0.5f;
+    if( mMode == TabBoxWindowsMode || mMode == TabBoxWindowsAlternativeMode )
         {
-        foreach( EffectWindow* w, windows.keys())
+        QHash< EffectWindow*, ItemInfo* >::const_iterator i;
+        for( i = windows.constBegin(); i != windows.constEnd(); ++i )
             {
-            if( w != selected_window )
-                w->addRepaintFull();
+            if( i.key() != selected_window )
+                i.key()->addRepaintFull();
             }
         // We don't unset the selected window so we have correct fading
         // But we do need to remove elevation status
@@ -509,7 +542,7 @@ void BoxSwitchEffect::setSelectedWindow( EffectWindow* w )
         effects->setElevatedWindow( selected_window, false );
         }
     selected_window = w;
-    if( selected_window )
+    if( selected_window && ( !mProxyActivated || mProxyShowText ) )
         thumbnailFrame.setText( selected_window->caption() );
     if( elevate_window && w )
         {
@@ -544,7 +577,7 @@ void BoxSwitchEffect::calculateFrameSize()
     {
     int itemcount;
 
-    if( mMode == TabBoxWindowsMode )
+    if( mMode == TabBoxWindowsMode || mMode == TabBoxWindowsAlternativeMode )
         {
         itemcount = original_windows.count();
         item_max_size.setWidth( 200 );
@@ -570,10 +603,12 @@ void BoxSwitchEffect::calculateFrameSize()
         }
     frame_area.setHeight( item_max_size.height() +
             separator_height + text_area.height());
+    if( mProxyActivated && !mProxyShowText )
+        frame_area.setHeight( item_max_size.height() );
     text_area.setWidth( frame_area.width() );
 
     frame_area.moveTo( screenr.x() + ( screenr.width() - frame_area.width()) / 2,
-        screenr.y() + ( screenr.height() - frame_area.height()) / 2 );
+        screenr.y() + ( screenr.height() - frame_area.height()) / 2 * mPositioningFactor * 2 );
     text_area.moveTo( frame_area.x(),
                       frame_area.y() + item_max_size.height() + separator_height);
 
@@ -582,11 +617,11 @@ void BoxSwitchEffect::calculateFrameSize()
 
 void BoxSwitchEffect::calculateItemSizes()
     {
-    if( mMode == TabBoxWindowsMode )
+    if( mMode == TabBoxWindowsMode || mMode == TabBoxWindowsAlternativeMode )
         {
         qDeleteAll( windows );
         windows.clear();
-        if( mAnimateSwitch )
+        if( (mAnimateSwitch && !mProxyActivated) || (mProxyActivated && mProxyAnimateSwitch) )
             {
             int index = original_windows.indexOf( effects->currentTabBoxWindow() );
             int leftIndex = index;
@@ -811,7 +846,7 @@ void BoxSwitchEffect::paintWindowThumbnail( EffectWindow* w )
             }
 
         // paint one part of the thumbnail
-        effects->drawWindow( w,
+        effects->paintWindow( w,
             PAINT_WINDOW_OPAQUE | PAINT_WINDOW_TRANSFORMED,
             windows[ w ]->thumbnail, data );
 
@@ -860,7 +895,7 @@ void BoxSwitchEffect::paintWindowThumbnail( EffectWindow* w )
             windows[ w ]->thumbnail, w,
             secondThumbnail.adjusted( highlight_margin, highlight_margin, -highlight_margin, -highlight_margin ),
             Qt::KeepAspectRatio );
-        effects->drawWindow( w,
+        effects->paintWindow( w,
             PAINT_WINDOW_OPAQUE | PAINT_WINDOW_TRANSFORMED,
             windows[ w ]->thumbnail, data );
         }
@@ -932,9 +967,12 @@ void BoxSwitchEffect::paintWindowThumbnail( EffectWindow* w )
         }
     else
         {
+        int mask = PAINT_WINDOW_OPAQUE | PAINT_WINDOW_TRANSFORMED;
+        if (!mAnimateSwitch)
+            mask |= PAINT_WINDOW_LANCZOS;
         effects->drawWindow( w,
-            PAINT_WINDOW_OPAQUE | PAINT_WINDOW_TRANSFORMED,
-            windows[ w ]->thumbnail, data );
+                 mask,
+                windows[ w ]->thumbnail, data );
         }
     }
 
@@ -1035,6 +1073,48 @@ void BoxSwitchEffect::paintWindowIcon( EffectWindow* w )
     windows[ w ]->iconFrame->render( infiniteRegion(), 1.0, 0.75 );
     }
 
+void* BoxSwitchEffect::proxy()
+    {
+    return &mProxy;
+    }
+
+void BoxSwitchEffect::activateFromProxy( int mode, bool animate, bool showText, float positioningFactor )
+    {
+    if( !mActivated )
+        {
+        mProxyActivated = true;
+        mProxyAnimateSwitch = animate;
+        mProxyShowText = showText;
+        mPositioningFactor = positioningFactor;
+        thumbnailFrame.setText(" ");
+        if( ( mode == TabBoxWindowsMode ) || ( mode == TabBoxWindowsAlternativeMode ) )
+            {
+            if( effects->currentTabBoxWindowList().count() > 0 )
+                {
+                mMode = mode;
+                effects->refTabBox();
+                highlight_is_set = false;
+                animation = false;
+                scheduled_directions.clear();
+                right_window = 0;
+                setActive();
+                }
+            }
+        else if( mode == TabBoxDesktopListMode || mode == TabBoxDesktopMode )
+            { // DesktopMode
+            if( effects->currentTabBoxDesktopList().count() > 0 )
+                {
+                mMode = mode;
+                painting_desktop = 0;
+                effects->refTabBox();
+                setActive();
+                }
+            }
+        if( !mActivated )
+            mProxyActivated = false;
+        }
+    }
+
 BoxSwitchEffect::ItemInfo::ItemInfo()
     : iconFrame( NULL )
     {
