@@ -51,6 +51,7 @@ Rules::Rules()
     , maxsizerule( UnusedForceRule )
     , opacityactiverule( UnusedForceRule )
     , opacityinactiverule( UnusedForceRule )
+    , tilingoptionrule( UnusedForceRule )
     , ignorepositionrule( UnusedForceRule )
     , desktoprule( UnusedSetRule )
     , typerule( UnusedForceRule )
@@ -60,6 +61,7 @@ Rules::Rules()
     , shaderule( UnusedSetRule )
     , skiptaskbarrule( UnusedSetRule )
     , skippagerrule( UnusedSetRule )
+    , skipswitcherrule( UnusedSetRule )
     , aboverule( UnusedSetRule )
     , belowrule( UnusedSetRule )
     , fullscreenrule( UnusedSetRule )
@@ -153,6 +155,7 @@ void Rules::readFromCfg( const KConfigGroup& cfg )
     READ_FORCE_RULE( opacityinactive,, 0);
     if( opacityinactive < 0 || opacityinactive > 100 )
         opacityinactive = 100;
+    READ_FORCE_RULE( tilingoption,, 0 );
     READ_FORCE_RULE( ignoreposition,, false);
     READ_SET_RULE( desktop,,0  );
     type = readType( cfg, "type" );
@@ -163,6 +166,7 @@ void Rules::readFromCfg( const KConfigGroup& cfg )
     READ_SET_RULE( shade,, false);
     READ_SET_RULE( skiptaskbar,, false);
     READ_SET_RULE( skippager,, false);
+    READ_SET_RULE( skipswitcher,, false);
     READ_SET_RULE( above,, false);
     READ_SET_RULE( below,, false);
     READ_SET_RULE( fullscreen,, false);
@@ -241,6 +245,7 @@ void Rules::write( KConfigGroup& cfg ) const
     WRITE_FORCE_RULE( maxsize, );
     WRITE_FORCE_RULE( opacityactive, );
     WRITE_FORCE_RULE( opacityinactive, );
+    WRITE_FORCE_RULE( tilingoption, );
     WRITE_FORCE_RULE( ignoreposition, );
     WRITE_SET_RULE( desktop, );
     WRITE_FORCE_RULE( type, int );
@@ -250,6 +255,7 @@ void Rules::write( KConfigGroup& cfg ) const
     WRITE_SET_RULE( shade, );
     WRITE_SET_RULE( skiptaskbar, );
     WRITE_SET_RULE( skippager, );
+    WRITE_SET_RULE( skipswitcher, );
     WRITE_SET_RULE( above, );
     WRITE_SET_RULE( below, );
     WRITE_SET_RULE( fullscreen, );
@@ -280,6 +286,7 @@ bool Rules::isEmpty() const
         && maxsizerule == UnusedForceRule
         && opacityactiverule == UnusedForceRule
         && opacityinactiverule == UnusedForceRule
+        && tilingoptionrule == UnusedForceRule
         && ignorepositionrule == UnusedForceRule
         && desktoprule == UnusedSetRule
         && typerule == UnusedForceRule
@@ -289,6 +296,7 @@ bool Rules::isEmpty() const
         && shaderule == UnusedSetRule
         && skiptaskbarrule == UnusedSetRule
         && skippagerrule == UnusedSetRule
+        && skipswitcherrule == UnusedSetRule
         && aboverule == UnusedSetRule
         && belowrule == UnusedSetRule
         && fullscreenrule == UnusedSetRule
@@ -490,6 +498,11 @@ bool Rules::update( Client* c )
         updated = updated || skippager != c->skipPager();
         skippager = c->skipPager();
         }
+    if( skipswitcherrule == ( SetRule )Remember)
+        {
+        updated = updated || skipswitcher != c->skipSwitcher();
+        skipswitcher = c->skipSwitcher();
+        }
     if( aboverule == ( SetRule )Remember)
         {
         updated = updated || above != c->keepAbove();
@@ -576,6 +589,7 @@ APPLY_FORCE_RULE( maxsize, MaxSize, QSize )
 APPLY_FORCE_RULE( opacityactive, OpacityActive, int )
 APPLY_FORCE_RULE( opacityinactive, OpacityInactive, int )
 APPLY_FORCE_RULE( ignoreposition, IgnorePosition, bool )
+APPLY_FORCE_RULE( tilingoption, TilingOption, int )
 
 // the cfg. entry needs to stay named the say for backwards compatibility
 bool Rules::applyIgnoreGeometry( bool& ignore ) const
@@ -616,6 +630,7 @@ bool Rules::applyShade( ShadeMode& sh, bool init ) const
 
 APPLY_RULE( skiptaskbar, SkipTaskbar, bool )
 APPLY_RULE( skippager, SkipPager, bool )
+APPLY_RULE( skipswitcher, SkipSwitcher, bool )
 APPLY_RULE( above, KeepAbove, bool )
 APPLY_RULE( below, KeepBelow, bool )
 APPLY_RULE( fullscreen, FullScreen, bool )
@@ -672,6 +687,7 @@ void Rules::discardUsed( bool withdrawn )
     DISCARD_USED_FORCE_RULE( maxsize );
     DISCARD_USED_FORCE_RULE( opacityactive );
     DISCARD_USED_FORCE_RULE( opacityinactive );
+    DISCARD_USED_FORCE_RULE( tilingoption );
     DISCARD_USED_FORCE_RULE( ignoreposition );
     DISCARD_USED_SET_RULE( desktop );
     DISCARD_USED_FORCE_RULE( type );
@@ -681,6 +697,7 @@ void Rules::discardUsed( bool withdrawn )
     DISCARD_USED_SET_RULE( shade );
     DISCARD_USED_SET_RULE( skiptaskbar );
     DISCARD_USED_SET_RULE( skippager );
+    DISCARD_USED_SET_RULE( skipswitcher );
     DISCARD_USED_SET_RULE( above );
     DISCARD_USED_SET_RULE( below );
     DISCARD_USED_SET_RULE( fullscreen );
@@ -781,6 +798,7 @@ CHECK_FORCE_RULE( MinSize, QSize )
 CHECK_FORCE_RULE( MaxSize, QSize )
 CHECK_FORCE_RULE( OpacityActive, int )
 CHECK_FORCE_RULE( OpacityInactive, int )
+CHECK_FORCE_RULE( TilingOption, int )
 CHECK_FORCE_RULE( IgnorePosition, bool )
 
 bool WindowRules::checkIgnoreGeometry( bool ignore ) const
@@ -804,6 +822,7 @@ CHECK_RULE( Minimize, bool )
 CHECK_RULE( Shade, ShadeMode )
 CHECK_RULE( SkipTaskbar, bool )
 CHECK_RULE( SkipPager, bool )
+CHECK_RULE( SkipSwitcher, bool )
 CHECK_RULE( KeepAbove, bool )
 CHECK_RULE( KeepBelow, bool )
 CHECK_RULE( FullScreen, bool )
@@ -856,6 +875,7 @@ void Client::applyWindowRules()
     setShade( shadeMode());
     setSkipTaskbar( skipTaskbar(), true );
     setSkipPager( skipPager());
+    setSkipSwitcher( skipSwitcher());
     setKeepAbove( keepAbove());
     setKeepBelow( keepBelow());
     setFullScreen( isFullScreen(), true );
