@@ -76,6 +76,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "client.h"
 #include "deleted.h"
 #include "effects.h"
+#include "lanczosfilter.h"
 
 #include <kephal/screens.h>
 
@@ -92,11 +93,13 @@ Scene::Scene( Workspace* ws )
     : wspace( ws )
     , has_waitSync( false )
     , selfCheckDone( false )
+    , lanczos_filter( new LanczosFilter() )
     {
     }
     
 Scene::~Scene()
     {
+    delete lanczos_filter;
     }
 
 // returns mask and possibly modified region
@@ -186,7 +189,7 @@ void Scene::paintGenericScreen( int orig_mask, ScreenPaintData )
     foreach( Window* w, stacking_order ) // bottom to top
         {
         WindowPrePaintData data;
-        data.mask = orig_mask | ( w->isOpaque() ? PAINT_WINDOW_OPAQUE : PAINT_WINDOW_TRANSLUCENT );
+        data.mask = orig_mask | ( w->isOpaque()? PAINT_WINDOW_OPAQUE : PAINT_WINDOW_TRANSLUCENT );
         w->resetPaintingEnabled();
         data.paint = infiniteRegion(); // no clipping, so doesn't really matter
         data.clip = QRegion();
@@ -321,7 +324,10 @@ void Scene::finalPaintWindow( EffectWindowImpl* w, int mask, QRegion region, Win
 // will be eventually called from drawWindow()
 void Scene::finalDrawWindow( EffectWindowImpl* w, int mask, QRegion region, WindowPaintData& data )
     {
-    w->sceneWindow()->performPaint( mask, region, data );
+    if (mask & PAINT_WINDOW_LANCZOS )
+        lanczos_filter->performPaint( w, mask, region, data );
+    else
+        w->sceneWindow()->performPaint( mask, region, data );
     }
 
 QList< QPoint > Scene::selfCheckPoints() const
@@ -426,6 +432,8 @@ bool Scene::Window::isVisible() const
         return false;
     if( !toplevel->isOnCurrentDesktop())
         return false;
+    if( !toplevel->isOnCurrentActivity())
+        return false;
     if( Client* c = dynamic_cast< Client* >( toplevel ))
         return c->isShown( true );
     return true; // Unmanaged is always visible
@@ -452,6 +460,8 @@ void Scene::Window::resetPaintingEnabled()
         disable_painting |= PAINT_DISABLED_BY_DELETE;
     if( !toplevel->isOnCurrentDesktop())
         disable_painting |= PAINT_DISABLED_BY_DESKTOP;
+    if( !toplevel->isOnCurrentActivity())
+        disable_painting |= PAINT_DISABLED_BY_ACTIVITY;
     if( Client* c = dynamic_cast< Client* >( toplevel ))
         {
         if( c->isMinimized() )
@@ -478,7 +488,7 @@ WindowQuadList Scene::Window::buildQuads( bool force ) const
     if( cached_quad_list != NULL && !force )
         return *cached_quad_list;
     WindowQuadList ret;
-    if( toplevel->clientPos() == QPoint( 0, 0 ) && toplevel->clientSize() == toplevel->size())
+    if( toplevel->clientPos() == QPoint( 0, 0 ) && toplevel->clientSize() == toplevel->visibleRect().size())
         ret = makeQuads( WindowQuadContents, shape()); // has no decoration
     else
         {
