@@ -73,6 +73,7 @@ class SceneOpenGL
             public:
                 GLXFBConfig fbconfig;
                 int bind_texture_format;
+                int texture_targets;
                 int y_inverted;
                 int mipmap;
             };
@@ -123,7 +124,8 @@ class SceneOpenGL::Texture
     private:
         void init();
 
-        GLXPixmap bound_glxpixmap; // the glx pixmap the texture is bound to, only for tfp_mode
+        GLXPixmap glxpixmap; // the glx pixmap the texture is bound to, only for tfp_mode
+        bool damaged;
     };
 
 class SceneOpenGL::Window
