@@ -48,6 +48,7 @@
 #include <Plasma/AbstractToolBox>
 #include <Plasma/View>
 #include <Plasma/PaintUtils>
+#include <Plasma/WindowEffects>
 
 #include <kephal/screens.h>
 
@@ -118,7 +119,8 @@ Panel::Panel(QObject *parent, const QVariantList &args)
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeUpdated()));
 
     connect(this, SIGNAL(appletRemoved(Plasma::Applet*)),
-            this, SLOT(appletRemoved(Plasma::Applet*)));
+            this, SLOT(appletWasRemoved(Plasma::Applet*)));
+    setContainmentType(Containment::PanelContainment);
 }
 
 Panel::~Panel()
@@ -127,18 +129,15 @@ Panel::~Panel()
 
 void Panel::init()
 {
-    setContainmentType(Containment::PanelContainment);
-
     Containment::init();
     //FIXME: This should be enabled, but in that case proxywidgets won't get rendered
     //setFlag(ItemClipsChildrenToShape, true);
 
     m_layout = new QGraphicsLinearLayout(this);
-    m_layout->setContentsMargins(0, 0, 0, 0);
     m_layout->setSpacing(4);
     m_layout->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
-    setLayout(m_layout);
     updateBorders(geometry().toRect());
+    setLayout(m_layout);
     m_layout->setMaximumSize(size());
 
     KConfigGroup cg = config("Configuration");
@@ -298,17 +297,23 @@ void Panel::layoutApplet(Plasma::Applet* applet, const QPointF &pos)
     //FIXME: there must be some beter way to do this rather than this rather error prone arbitrary wait
     m_lastSpaceTimer->start(2000);
 
-    connect(applet, SIGNAL(sizeHintChanged(Qt::SizeHint)), this, SLOT(updateSize()));
+    connect(applet, SIGNAL(sizeHintChanged(Qt::SizeHint)), this, SLOT(delayedUpdateSize()), Qt::UniqueConnection);
 }
 
-void Panel::appletRemoved(Plasma::Applet* applet)
+void Panel::delayedUpdateSize()
 {
+    m_lastResizedApplet = qobject_cast<Plasma::Applet *>(sender());
+    QTimer::singleShot(0, this, SLOT(updateSize()));
+}
+
+void Panel::appletWasRemoved(Plasma::Applet* applet)
+{
+    disconnect(applet, SIGNAL(sizeHintChanged(Qt::SizeHint)), this, SLOT(delayedUpdateSize()));
+
     if (!m_layout) {
         return;
     }
 
-    connect(applet, SIGNAL(sizeHintChanged(Qt::SizeHint)), this, SLOT(updateSize()));
-
     m_layout->removeItem(applet);
 
     //shrink the panel if possible
@@ -317,14 +322,15 @@ void Panel::appletRemoved(Plasma::Applet* applet)
     } else {
         resize(size().width(), size().height() - applet->size().height());
     }
-    m_layout->setMaximumSize(size());
 
+    m_layout->setMaximumSize(size());
     m_lastSpaceTimer->start(200);
 }
 
 void Panel::updateSize()
 {
     Plasma::Applet *applet = qobject_cast<Plasma::Applet *>(sender());
+    applet = m_lastResizedApplet.data();
 
     if (m_canResize && applet) {
         if (formFactor() == Plasma::Horizontal) {
@@ -348,7 +354,7 @@ void Panel::updateSize()
     }
 }
 
-void Panel::updateBorders(const QRect &geom, bool themeChange)
+void Panel::updateBorders(const QRect &geom, bool inPaintEvent)
 {
     Plasma::Location loc = location();
     FrameSvg::EnabledBorders enabledBorders = FrameSvg::AllBorders;
@@ -432,43 +438,34 @@ void Panel::updateBorders(const QRect &geom, bool themeChange)
     if (m_layout) {
         switch (location()) {
         case LeftEdge:
-            rightWidth = qMin(rightWidth, qMax(qreal(1), size().width() - KIconLoader::SizeMedium));
+            rightWidth = qMin(rightWidth, qMax(qreal(2), size().width() - KIconLoader::SizeMedium));
             break;
         case RightEdge:
-            leftWidth = qMin(leftWidth, qMax(qreal(1), size().width() - KIconLoader::SizeMedium));
+            leftWidth = qMin(leftWidth, qMax(qreal(2), size().width() - KIconLoader::SizeMedium));
             break;
         case TopEdge:
-            bottomHeight = qMin(bottomHeight, qMax(qreal(1), size().height() - KIconLoader::SizeMedium));
+            bottomHeight = qMin(bottomHeight, qMax(qreal(2), size().height() - KIconLoader::SizeMedium));
             break;
         case BottomEdge:
-            topHeight = qMin(topHeight, qMax(qreal(1), size().height() - KIconLoader::SizeMedium));
+            topHeight = qMin(topHeight, qMax(qreal(2), size().height() - KIconLoader::SizeMedium));
             break;
         default:
             break;
         }
-        qreal oldLeft = leftWidth;
-        qreal oldTop = topHeight;
-        qreal oldRight = rightWidth;
-        qreal oldBottom = bottomHeight;
-
-        if (themeChange) {
-            m_layout->getContentsMargins(&oldLeft, &oldTop, &oldRight, &oldBottom);
-        }
 
         m_layout->setContentsMargins(leftWidth, topHeight, rightWidth, bottomHeight);
 
-        m_layout->invalidate();
-        resize(preferredSize());
+        if (!inPaintEvent) {
+            resize(preferredSize());
+        }
     }
-
-    update();
 }
 
 void Panel::constraintsEvent(Plasma::Constraints constraints)
 {
-    m_maskDirty = true;
-
     if (constraints & Plasma::FormFactorConstraint) {
+        m_maskDirty = true;
+
         Plasma::FormFactor form = formFactor();
         Qt::Orientation layoutDirection = form == Plasma::Vertical ? Qt::Vertical : Qt::Horizontal;
         // create or set up our layout!
@@ -480,6 +477,7 @@ void Panel::constraintsEvent(Plasma::Constraints constraints)
 
     //we need to know if the width or height is 100%
     if (constraints & Plasma::LocationConstraint || constraints & Plasma::SizeConstraint) {
+        m_maskDirty = true;
         m_currentSize = geometry().size().toSize();
         QRectF screenRect = screen() >= 0 ? Kephal::ScreenUtils::screenGeometry(screen()) :
                                             geometry();
@@ -533,6 +531,7 @@ void Panel::constraintsEvent(Plasma::Constraints constraints)
             m_configureAction->setVisible(unlocked);
         }
 
+        m_maskDirty = true;
         updateBorders(geometry().toRect());
     }
 }
@@ -545,7 +544,7 @@ void Panel::saveState(KConfigGroup &config) const
 
 void Panel::themeUpdated()
 {
-    updateBorders(geometry().toRect(), true);
+    updateBorders(geometry().toRect());
 }
 
 void Panel::paintInterface(QPainter *painter,
@@ -562,15 +561,19 @@ void Panel::paintInterface(QPainter *painter,
     QRect viewGeom;
     if (containmentOpt && containmentOpt->view) {
         viewGeom = containmentOpt->view->geometry();
+    } else {
+        viewGeom = m_lastViewGeom;
     }
 
     if (m_maskDirty || m_lastViewGeom != viewGeom) {
         m_maskDirty = false;
         m_lastViewGeom = viewGeom;
 
-        updateBorders(viewGeom);
+        updateBorders(viewGeom, true);
         if (containmentOpt && containmentOpt->view && !m_background->mask().isEmpty()) {
-            containmentOpt->view->setMask(m_background->mask());
+            const QRegion mask = m_background->mask();
+            containmentOpt->view->setMask(mask);
+            Plasma::WindowEffects::enableBlurBehind(containmentOpt->view->winId(), true, mask);
         }
     }
 
@@ -685,7 +688,7 @@ void Panel::restore(KConfigGroup &group)
             unoderedApplets.append(applet);
         }
 
-        connect(applet, SIGNAL(sizeHintChanged(Qt::SizeHint)), this, SLOT(updateSize()));
+        connect(applet, SIGNAL(sizeHintChanged(Qt::SizeHint)), this, SLOT(delayedUpdateSize()), Qt::UniqueConnection);
     }
 
     foreach (Applet *applet, oderedApplets) {
