@@ -1196,6 +1196,31 @@ void PanelView::updateStruts()
     recreateUnhideTrigger();
 }
 
+bool PanelView:: migratedFrom(int screenId) const
+{
+    KConfigGroup cg = config();
+    QList<int> migrations;
+    migrations = cg.readEntry("Migrations", migrations);
+    return migrations.contains(screenId);
+}
+
+void PanelView::migrateTo(int screenId)
+{
+    KConfigGroup cg = config();
+    QList<int> migrations;
+    migrations = cg.readEntry("Migrations", migrations);
+
+    const int index = migrations.indexOf(screenId);
+    if (index == -1) {
+        migrations.append(screenId);
+    } else {
+        migrations = migrations.mid(0, migrations.length() - index - 1);
+    }
+
+    cg.writeEntry("Migrations", migrations);
+    setScreen(screenId);
+}
+
 void PanelView::enterEvent(QEvent *event)
 {
     // allow unhiding to happen again even if we were delay-unhidden
@@ -1396,6 +1421,7 @@ void PanelView::unhide(bool destroyTrigger)
     //avoid hide-show loops
     if (m_visibilityMode == LetWindowsCover) {
         m_triggerEntered = true;
+        KWindowSystem::clearState(winId(), NET::KeepBelow);
         KWindowSystem::raiseWindow(winId());
         QTimer::singleShot(0, this, SLOT(resetTriggerEnteredSuppression()));
     }
@@ -1447,6 +1473,7 @@ void PanelView::startAutoHide()
     }
 
     if (m_visibilityMode == LetWindowsCover) {
+        KWindowSystem::setState(winId(), NET::KeepBelow);
         KWindowSystem::lowerWindow(winId());
         createUnhideTrigger();
     } else {
