@@ -55,9 +55,10 @@ DeviceItem::DeviceItem(const QString &udi, QGraphicsWidget *parent)
       m_udi(udi),
       m_hovered(false),
       m_mounted(false),
+      m_safelyRemovable(true),
+      m_state(DeviceItem::Idle),
       m_labelFade(0),
-      m_barFade(0),
-      m_iconFade(0)
+      m_barFade(0)
 {
     setAcceptHoverEvents(true);
     setCacheMode(DeviceCoordinateCache);
@@ -92,12 +93,13 @@ DeviceItem::DeviceItem(const QString &udi, QGraphicsWidget *parent)
     m_nameLabel->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
     m_nameLabel->setPreferredWidth(0);
     m_nameLabel->nativeWidget()->setWordWrap(false);
+    m_nameLabel->setAcceptedMouseButtons(0);
 
     m_descriptionLabel = new Plasma::Label(this);
     m_descriptionLabel->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
     m_descriptionLabel->setPreferredWidth(0);
     m_descriptionLabel->nativeWidget()->setWordWrap(false);
-    m_descriptionLabel->setAttribute(Qt::WA_TransparentForMouseEvents, true);
+    m_descriptionLabel->setAcceptedMouseButtons(0);
     QFont font = m_descriptionLabel->font();
     font.setPointSize(KGlobalSettings::smallestReadableFont().pointSize());
     font.setItalic(true);
@@ -111,7 +113,6 @@ DeviceItem::DeviceItem(const QString &udi, QGraphicsWidget *parent)
     capacityBarWidget->setAttribute(Qt::WA_TranslucentBackground);
     capacityBarWidget->setContinuous(true);
     m_capacityBar->setAcceptHoverEvents(false);
-    m_capacityBar->setMaximumHeight(12);
     m_capacityBar->setOpacity(0);
     info_layout->addItem(m_nameLabel);
     info_layout->addItem(m_descriptionLabel);
@@ -120,7 +121,6 @@ DeviceItem::DeviceItem(const QString &udi, QGraphicsWidget *parent)
     m_leftActionIcon = new Plasma::IconWidget(this);
     m_leftActionIcon->setMaximumSize(m_leftActionIcon->sizeFromIconSize(LEFTACTION_SIZE));
     m_leftActionIcon->setSizePolicy(QSizePolicy::Fixed,  QSizePolicy::Fixed);
-    m_leftActionIcon->setOpacity(0);
     connect(m_leftActionIcon, SIGNAL(clicked()), this, SLOT(leftActionClicked()));
 
     m_mainLayout->addItem(m_deviceIcon);
@@ -129,6 +129,7 @@ DeviceItem::DeviceItem(const QString &udi, QGraphicsWidget *parent)
     m_mainLayout->setAlignment(info_layout, Qt::AlignVCenter);
     m_mainLayout->addItem(m_leftActionIcon);
     m_mainLayout->setAlignment(m_leftActionIcon, Qt::AlignVCenter);
+    m_mainLayout->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
 
     m_busyWidget = new Plasma::BusyWidget(this);
     m_busyWidget->setMaximumSize(LEFTACTION_SIZE, LEFTACTION_SIZE);
@@ -264,27 +265,55 @@ bool DeviceItem::allowsCapacityBar() const
     return !(data(NotifierDialog::IsOpticalMedia).toBool() || data(NotifierDialog::IsEncryptedContainer).toBool());
 }
 
+bool DeviceItem::safelyRemovable() const
+{
+    return m_safelyRemovable;
+}
+
+void DeviceItem::setSafelyRemovable(const bool safe)
+{
+    if (m_safelyRemovable == safe) {
+        return;
+    }
+    m_safelyRemovable = safe;
+    updateTooltip();
+}
+
+void DeviceItem::updateTooltip()
+{
+    if (m_mounted) {
+        if (data(NotifierDialog::IsOpticalMedia).toBool()) {
+            m_leftActionIcon->setToolTip(i18n("Click to eject this disc."));
+        } else {
+            m_leftActionIcon->setToolTip(i18n("Click to safely remove this device."));
+        }
+        m_deviceIcon->setToolTip(i18n("It is currently <b>not safe</b> to remove this device: applications may be accessing it. Click the eject button to safely remove this device."));
+    } else {
+        m_leftActionIcon->setToolTip(i18n("Click to access this device from other applications."));
+        if (safelyRemovable()) {
+            m_deviceIcon->setToolTip(i18n("It is currently safe to remove this device."));
+        } else {
+            m_deviceIcon->setToolTip(i18n("It is currently <b>not safe</b> to remove this device: applications may be accessing other volumes on this device. Click the eject button on these other volumes to safely remove this device."));
+        }
+    }
+}
+
 void DeviceItem::setMounted(const bool mounted)
 {
     m_mounted = mounted;
-    if (m_mounted) {
-        m_leftActionIcon->setToolTip(i18n("Click to safely remove this device from the computer."));
-        m_deviceIcon->setToolTip(i18n("Device is plugged in and can be accessed by applications."));
-    } else {
-        m_leftActionIcon->setToolTip(i18n("Click to access this device from other applications."));
-        m_deviceIcon->setToolTip(i18n("Device is plugged in, but not mounted for access yet."));
-    }
+
+    updateTooltip();
 
     if (data(NotifierDialog::IsEncryptedContainer).toBool()) {
-        if (m_mounted && m_leftAction != Lock) {
+        if (m_mounted) {
             setLeftAction(Lock);
-        } else if (!m_mounted && m_leftAction != Unlock) {
+        } else {
             setLeftAction(Unlock);
         }
     } else {
-        if (m_mounted && m_leftAction != Umount) {
+        if (m_mounted) {
             setLeftAction(Umount);
-        } else if (!m_mounted && m_leftAction != Mount) {
+        } else {
             setLeftAction(Mount);
         }
     }
@@ -299,6 +328,7 @@ void DeviceItem::setMounted(const bool mounted)
 
 void DeviceItem::setLeftAction(DeviceItem::LeftActions action)
 {
+    kDebug() << "setting to" << action;
     m_leftAction = action;
     if (m_leftAction == Umount) {
         m_leftActionIcon->setIcon("media-eject");
@@ -329,34 +359,27 @@ void DeviceItem::setHovered(const bool hovered)
         if (!m_labelFade) {
             m_labelFade = Plasma::Animator::create(Plasma::Animator::FadeAnimation, this);
             m_barFade = Plasma::Animator::create(Plasma::Animator::FadeAnimation, this);
-            m_iconFade = Plasma::Animator::create(Plasma::Animator::FadeAnimation, this);
 
             m_labelFade->setTargetWidget(m_descriptionLabel);
             m_barFade->setTargetWidget(m_capacityBar);
-            m_iconFade->setTargetWidget(m_leftActionIcon);
 
             m_labelFade->setProperty("targetOpacity", 0);
             m_barFade->setProperty("targetOpacity", 0);
-            m_iconFade->setProperty("targetOpacity", 0);
         }
         qreal currentOpacity = m_descriptionLabel->opacity();
 
         m_labelFade->setProperty("startOpacity", currentOpacity);
         m_barFade->setProperty("startOpacity", currentOpacity);
-        m_iconFade->setProperty("startOpacity", currentOpacity);
 
         m_labelFade->start();
         m_barFade->start();
-        m_iconFade->start();
     }
 }
 
 void DeviceItem::setHoverDisplayOpacity(qreal opacity)
 {
     m_descriptionLabel->setOpacity(opacity);
-    m_leftActionIcon->setOpacity(opacity);
     m_capacityBar->setOpacity(opacity);
-
 }
 
 void DeviceItem::setFreeSpace(qulonglong freeSpace, qulonglong size)
@@ -392,27 +415,42 @@ void DeviceItem::setData(int key, const QVariant & value)
     }
 }
 
-void DeviceItem::setBusy()
+void DeviceItem::setState(DeviceItem::State state)
 {
-    if (m_busyWidgetTimer.isActive()) {
-        return;
+    m_state = state;
+
+    if (state == Idle) {
+        m_descriptionLabel->setText(description());
+
+        if (m_busyWidgetTimer.isActive()) {
+            m_busyWidgetTimer.stop();
+        }
+
+        if (m_busyWidget->isVisible()) {
+            m_busyWidget->hide();
+            m_mainLayout->removeItem(m_busyWidget);
+            m_mainLayout->addItem(m_leftActionIcon);
+            m_mainLayout->setAlignment(m_leftActionIcon, Qt::AlignVCenter);
+            m_leftActionIcon->show();
+        }
+    } else {
+        if (m_busyWidgetTimer.isActive()) {
+            return;
+        }
+        m_busyWidgetTimer.start(300);
+
+        if (state == Mounting) {
+            m_descriptionLabel->setText(i18nc("Accessing is a less technical word for Mounting; translation should be short and mean \'Currently mounting this device\'", "Accessing..."));
+        } else {
+            collapse();
+            m_descriptionLabel->setText(i18nc("Removing is a less technical word for Unmounting; translation shoud be short and mean \'Currently unmounting this device\'", "Removing..."));
+        }
     }
-    m_busyWidgetTimer.start(300);
 }
 
-void DeviceItem::setReady()
+DeviceItem::State DeviceItem::state() const
 {
-    if (m_busyWidgetTimer.isActive()) {
-        m_busyWidgetTimer.stop();
-    }
-
-    if (m_busyWidget->isVisible()) {
-        m_busyWidget->hide();
-        m_mainLayout->removeItem(m_busyWidget);
-        m_mainLayout->addItem(m_leftActionIcon);
-        m_mainLayout->setAlignment(m_leftActionIcon, Qt::AlignVCenter);
-        m_leftActionIcon->show();
-    }
+    return m_state;
 }
 
 void DeviceItem::triggerBusyWidget()
@@ -435,9 +473,10 @@ bool DeviceItem::eventFilter(QObject* obj, QEvent *event)
             case QEvent::GraphicsSceneHoverEnter:
                 emit highlightActionItem(item);
                 break;
-            case QEvent::GraphicsSceneMousePress: {
+            case QEvent::GraphicsSceneMouseRelease: {
                 QGraphicsSceneMouseEvent *e = static_cast<QGraphicsSceneMouseEvent *>(event);
-                if (e->button() == Qt::LeftButton) {
+                if (e->button() == Qt::LeftButton &&
+                    item->geometry().contains(item->mapToParent(e->pos()))) {
                     actionClicked(item);
                     return true;
                 }
@@ -501,15 +540,14 @@ void DeviceItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
 
 void DeviceItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 {
-    if (event->button() != Qt::LeftButton || !boundingRect().contains(event->pos())) {
-        return;
+    if (event->button() == Qt::LeftButton && boundingRect().contains(event->pos())) {
+        clicked();
     }
-    clicked();
 }
 
 void DeviceItem::clicked()
 {
-    if (m_actionsLayout->count() == 0) {
+    if ((m_actionsLayout->count() == 0) || (m_state == Umounting)) {
         return;
     }
 
