@@ -48,7 +48,7 @@
 using namespace Plasma;
 using namespace Notifier;
 
-static const char *DEFAULT_ICON_NAME = "device-notifier";
+static const char DEFAULT_ICON_NAME[] = "device-notifier";
 static const int NOTIFICATION_TIMEOUT = 10000;
 
 K_EXPORT_PLASMA_APPLET(devicenotifier, DeviceNotifier)
@@ -57,6 +57,7 @@ DeviceNotifier::DeviceNotifier(QObject *parent, const QVariantList &args)
     : Plasma::PopupApplet(parent, args),
       m_solidEngine(0),
       m_solidDeviceEngine(0),
+      m_deviceNotificationsEngine(0),
       m_dialog(0),
       m_numberItems(0),
       m_itemsValidity(0),
@@ -70,7 +71,7 @@ DeviceNotifier::DeviceNotifier(QObject *parent, const QVariantList &args)
     KGlobal::locale()->insertCatalog("solid_qt");
 
     // let's initialize the widget
-    setMinimumSize(graphicsWidget()->minimumSize());
+    resize(graphicsWidget()->minimumSize());
 }
 
 DeviceNotifier::~DeviceNotifier()
@@ -81,9 +82,10 @@ DeviceNotifier::~DeviceNotifier()
 void DeviceNotifier::init()
 {
     configChanged();
-    
+
     m_solidEngine = dataEngine("hotplug");
     m_solidDeviceEngine = dataEngine("soliddevice");
+    m_deviceNotificationsEngine = dataEngine("devicenotifications");
 
     connect(m_dialog, SIGNAL(deviceSelected()), this, SLOT(showPopup()));
 
@@ -96,15 +98,23 @@ void DeviceNotifier::init()
             this, SLOT(onSourceAdded(const QString&)));
     connect(m_solidEngine, SIGNAL(sourceRemoved(const QString&)),
             this, SLOT(onSourceRemoved(const QString&)));
+    connect(m_deviceNotificationsEngine, SIGNAL(sourceAdded(const QString&)),
+            this, SLOT(newNotification(const QString&)));
 
+    setStatus(Plasma::PassiveStatus);
     //feed the list with what is already reported by the engine
     fillPreviousDevices();
 
-    if (m_lastPlugged.count() == 0) {
-        setStatus(Plasma::PassiveStatus);
-    } else {
-        setStatus(Plasma::ActiveStatus);
-    }
+    m_iconTimer = new QTimer(this);
+    m_iconTimer->setSingleShot(true);
+    connect(m_iconTimer, SIGNAL(timeout()), this, SLOT(resetNotifierIcon()));
+}
+
+void DeviceNotifier::newNotification(const QString &source)
+{
+    DataEngine::Data data = m_deviceNotificationsEngine->query(source);
+    //TODO Check if we are actually displaying the device in question
+    showErrorMessage(data["error"].toString(), data["errorDetails"].toString(), data["udi"].toString());
 }
 
 void DeviceNotifier::configChanged()
@@ -136,7 +146,9 @@ void DeviceNotifier::fillPreviousDevices()
         // discard swap and partition tables
         Solid::Device parentDevice = device.parent();
         Solid::StorageDrive *drive = parentDevice.as<Solid::StorageDrive>();
-        if (drive && (!drive->isHotpluggable() && !drive->isRemovable())) {
+        const Solid::StorageVolume *volume = device.as<Solid::StorageVolume>();
+        if (drive && (!drive->isHotpluggable() && !drive->isRemovable()) &&
+            (volume->usage() == Solid::StorageVolume::FileSystem)) {
             deviceAdded(device, false);
         }
     }
@@ -148,9 +160,20 @@ void DeviceNotifier::fillPreviousDevices()
     m_fillingPreviousDevices = false;
 }
 
-void DeviceNotifier::changeNotifierIcon(const QString& name)
+void DeviceNotifier::changeNotifierIcon(const QString& name, uint timeout)
 {
+    m_iconTimer->stop();
     setPopupIcon(name.isNull() ? DEFAULT_ICON_NAME : name);
+    if (timeout) {
+        m_iconTimer->setInterval(timeout);
+        m_iconTimer->start();
+    }
+}
+
+void DeviceNotifier::resetNotifierIcon()
+{
+    changeNotifierIcon();
+    update();
 }
 
 void DeviceNotifier::popupEvent(bool show)
@@ -162,6 +185,7 @@ void DeviceNotifier::popupEvent(bool show)
     } else {
         m_dialog->collapseDevices();
     }
+    changeNotifierIcon();
 }
 
 void DeviceNotifier::dataUpdated(const QString &udi, Plasma::DataEngine::Data data)
@@ -255,14 +279,12 @@ void DeviceNotifier::notifyDevice(const QString &udi)
 {
     m_lastPlugged << udi;
 
-    setStatus(Plasma::NeedsAttentionStatus);
-
     if (!m_fillingPreviousDevices) {
         emit activate();
         showPopup(NOTIFICATION_TIMEOUT);
-        changeNotifierIcon("preferences-desktop-notification");
+        changeNotifierIcon("preferences-desktop-notification", NOTIFICATION_TIMEOUT);
         update();
-        QTimer::singleShot(NOTIFICATION_TIMEOUT, m_dialog, SLOT(resetNotifierIcon()));
+        setStatus(Plasma::NeedsAttentionStatus);
     } else {
         setStatus(Plasma::ActiveStatus);
     }
@@ -272,11 +294,11 @@ void DeviceNotifier::toolTipAboutToShow()
 {
     Plasma::ToolTipContent toolTip;
     if (m_lastPlugged.isEmpty()) {
-        toolTip.setSubText(i18n("No devices plugged in"));
+        toolTip.setSubText(i18n("No devices available."));
         toolTip.setImage(KIcon("device-notifier"));
     } else {
         Solid::Device device(m_lastPlugged.last());
-        toolTip.setSubText(i18n("Last plugged in device: %1", device.description()));
+        toolTip.setSubText(i18n("Most recent device: %1", device.description()));
         toolTip.setImage(KIcon(device.icon()));
     }
 
@@ -323,23 +345,50 @@ void DeviceNotifier::deviceAdded(const Solid::Device &device, bool hotplugged)
     bool visibility = cg.readEntry(udi, true);
 
     if (visibility || m_globalVisibility) {
-        m_dialog->insertDevice(udi);
+        // WORKAROUND: Some distributions do not set the HAL flag volume.ignore = true
+        // on partitions belonging to fixed devices; this causes issues since so far
+        // the code assumed this behavior. In particular some fixed devices might
+        // be added to the notifier twice: once as fixed devices (by fillPreviousDevices)
+        // and once as hotplugged devices (by onSourceAdded). The first time, however
+        // the dataengine might not be ready with the required data, so we need to
+        // make sure that the second time we reconnect the notifier to both engines in order to
+        // receive data through dataUpdated.
+        //
+        // Please note this is by no means a correct fix; it just avoids
+        // showing duplicate entries; mount/unmount actions typically will not
+        // work for such devices. A real fix seems to require a good amount of
+        // new code and would need to be throughoutly tested.
+        //
+        // TODO: Handling of fixed devices needs to be rethought for 4.6
 
-        if (hotplugged) {
-            notifyDevice(udi);
+        if (!m_lastPlugged.contains(udi)) {
+            m_dialog->insertDevice(udi);
+
+            if (hotplugged) {
+                notifyDevice(udi);
+            }
+
+            m_dialog->setDeviceData(udi, visibility, NotifierDialog::VisibilityRole);
+
+            m_lastPlugged << udi;
+        } else {
+            // Reconnect to both engines since now we can possibly
+            // correctly connect to the updates.
+            m_solidEngine->disconnectSource(udi, this);
+            m_solidDeviceEngine->disconnectSource(udi, this);
         }
 
-        m_dialog->setDeviceData(udi, visibility, NotifierDialog::VisibilityRole);
-
         m_solidEngine->connectSource(udi, this);
         m_solidDeviceEngine->connectSource(udi, this);
-
-        m_lastPlugged << udi;
     }
 
-    if (!visibility) {
+    if (!visibility && !m_hiddenDevices.contains(udi)) {
         m_hiddenDevices << udi;
     }
+
+    if (visibility) {
+        setStatus(Plasma::ActiveStatus);
+    }
 }
 
 void DeviceNotifier::onSourceRemoved(const QString &udi)
@@ -378,30 +427,46 @@ void DeviceNotifier::createConfigurationInterface(KConfigDialog *parent)
     m_deviceActionsWidget = new KCModuleProxy("solid-actions");
     m_autoMountingWidget = new KCModuleProxy("device_automounter_kcm");
 
-    parent->addPage(configurationWidget, i18nc("General options page", "General"), icon());
-    parent->addPage(m_deviceActionsWidget, m_deviceActionsWidget->moduleInfo().moduleName(), 
+    parent->addPage(configurationWidget, i18n("Display"), icon());
+    parent->addPage(m_deviceActionsWidget, m_deviceActionsWidget->moduleInfo().moduleName(),
                     m_deviceActionsWidget->moduleInfo().icon());
-    parent->addPage(m_autoMountingWidget, m_autoMountingWidget->moduleInfo().moduleName(),
+    parent->addPage(m_autoMountingWidget, i18n("Automounting"),
                     m_autoMountingWidget->moduleInfo().icon());
 
     parent->setButtons( KDialog::Ok | KDialog::Cancel);
     connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
 
-    m_configurationUi.showDevices->setCurrentIndex(m_showDevices);
+    switch (m_showDevices) {
+        case RemovableOnly:
+            m_configurationUi.removableDevices->setChecked(true);
+            break;
+        case NonRemovableOnly:
+            m_configurationUi.nonRemovableDevices->setChecked(true);
+            break;
+        case AllDevices:
+            m_configurationUi.allDevices->setChecked(true);
+            break;
+    }
 }
 
 void DeviceNotifier::configAccepted()
 {
     KConfigGroup cg = config();
 
-    m_showDevices = m_configurationUi.showDevices->currentIndex();
+    if (m_configurationUi.allDevices->isChecked()) {
+        m_showDevices = AllDevices;
+    } else if (m_configurationUi.nonRemovableDevices->isChecked()) {
+        m_showDevices = NonRemovableOnly;
+    } else {
+        m_showDevices = RemovableOnly;
+    }
 
     cg.writeEntry("ShowDevices", m_showDevices);
 
     //Save the configurations of the embedded KCMs
     m_deviceActionsWidget->save();
     m_autoMountingWidget->save();
-    
+
     emit configNeedsSaving();
 
     resetDevices();
@@ -431,9 +496,12 @@ void DeviceNotifier::setGlobalVisibility(bool visibility)
     resetDevices();
 }
 
-void DeviceNotifier::showErrorMessage(const QString &message)
+void DeviceNotifier::showErrorMessage(const QString &message, const QString &details, const QString &udi)
 {
-    showMessage(KIcon("dialog-error"), message, ButtonOk);
+    m_dialog->showStatusBarMessage(message, details, udi);
+    showPopup(NOTIFICATION_TIMEOUT);
+    changeNotifierIcon("dialog-error", NOTIFICATION_TIMEOUT);
+    update();
 }
 
 bool DeviceNotifier::areThereHiddenDevices()
