@@ -1,5 +1,6 @@
 /***************************************************************************
  *   Copyright (C) 2008 Marco Martin <notmart@gmail.com>                   *
+ *   Copyright (C) 2010 Davide Bettio <davide.bettio@kdemail.net>          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -19,14 +20,12 @@
 
 #include "webbrowser.h"
 
-#include <limits.h>
-
 #include <QGraphicsLinearLayout>
-#include <QGraphicsProxyWidget>
-#include <QGraphicsSceneMouseEvent>
-#include <QStandardItemModel>
 #include <QModelIndex>
-#include <QAction>
+#include <QNetworkReply>
+#include <QPainter>
+#include <QScrollBar>
+#include <QStandardItemModel>
 #include <QTimer>
 #include <QTreeView>
 #include <QWebPage>
@@ -42,17 +41,26 @@
 #include <KMessageBox>
 #include <KConfigDialog>
 #include <KHistoryComboBox>
+#include <KWebPage>
+#include <kwebwallet.h>
+#include <KStandardDirs>
 
-#include <Plasma/ComboBox>
+#include <Plasma/Animation>
 #include <Plasma/IconWidget>
-#include <Plasma/LineEdit>
-#include <Plasma/Meter>
 #include <Plasma/WebView>
 #include <Plasma/TreeView>
+#include <Plasma/PushButton>
 #include <Plasma/Slider>
 
 #include "bookmarksdelegate.h"
 #include "bookmarkitem.h"
+#include "webviewoverlay.h"
+#include "browserhistorycombobox.h"
+#include "browsermessagebox.h"
+#include "errorpage.h"
+#include "webbrowserpage.h"
+
+using Plasma::MessageButton;
 
 WebBrowser::WebBrowser(QObject *parent, const QVariantList &args)
         : Plasma::PopupApplet(parent, args),
@@ -62,13 +70,15 @@ WebBrowser::WebBrowser(QObject *parent, const QVariantList &args)
           m_completion(0),
           m_bookmarkManager(0),
           m_bookmarkModel(0),
-          m_autoRefreshTimer(0),
-          m_graphicsWidget(0),
-          m_historyCombo(0)
+          m_autoRefreshTimer(0)
 {
     setHasConfigurationInterface(true);
     setAspectRatioMode(Plasma::IgnoreAspectRatio);
 
+    m_historyCombo = 0;
+    m_graphicsWidget = 0;
+    m_webOverlay = 0;
+
     m_layout = 0;
     resize(500,500);
     if (!args.isEmpty()) {
@@ -83,9 +93,6 @@ QGraphicsWidget *WebBrowser::graphicsWidget()
         return m_graphicsWidget;
     }
 
-    KConfigGroup cg = config();
-
-
     m_layout = new QGraphicsLinearLayout(Qt::Vertical);
     m_toolbarLayout = new QGraphicsLinearLayout(Qt::Horizontal);
     m_statusbarLayout = new QGraphicsLinearLayout(Qt::Horizontal);
@@ -94,7 +101,7 @@ QGraphicsWidget *WebBrowser::graphicsWidget()
     m_forward = addTool("go-next", m_toolbarLayout);
 
     m_nativeHistoryCombo = new KHistoryComboBox();
-    m_historyCombo = new Plasma::ComboBox(this);
+    m_historyCombo = new Plasma::BrowserHistoryComboBox(this);
     m_historyCombo->setNativeWidget(m_nativeHistoryCombo);
     m_historyCombo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
     m_historyCombo->setZValue(999);
@@ -110,12 +117,11 @@ QGraphicsWidget *WebBrowser::graphicsWidget()
 
     m_layout->addItem(m_toolbarLayout);
 
-
     m_browser = new Plasma::WebView(this);
+    m_browser->setPage(new WebBrowserPage(this));
     m_browser->setPreferredSize(400, 400);
-    m_browser->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-    m_browser->setDragToScroll(cg.readEntry("DragToScroll", false));
 
+    m_browser->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
 
     m_layout->addItem(m_browser);
 
@@ -125,15 +131,13 @@ QGraphicsWidget *WebBrowser::graphicsWidget()
     bookmarksModelInit();
 
     m_bookmarksView = new Plasma::TreeView(this);
-    m_bookmarksView->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+    m_bookmarksView->setZValue(1);
+    m_bookmarksView->nativeWidget()->setAttribute(Qt::WA_NoSystemBackground, false);
+    m_bookmarksView->nativeWidget()->verticalScrollBar()->setStyle(QApplication::style());
+    m_bookmarksView->nativeWidget()->horizontalScrollBar()->setStyle(QApplication::style());
     m_bookmarksView->setModel(m_bookmarkModel);
     m_bookmarksView->nativeWidget()->setHeaderHidden(true);
-    //m_bookmarksView->nativeWidget()->viewport()->setAutoFillBackground(false);
-    m_bookmarksView->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
     m_bookmarksView->hide();
-    //FIXME: this is probably a Qt bug, QGraphicslayout always behaves as the hidden element is present, unlike QLayouts
-    m_bookmarksView->setMaximumHeight(0);
-    m_layout->addItem(m_bookmarksView);
 
     m_bookmarksDelegate = new BookmarksDelegate(this);
     m_bookmarksView->nativeWidget()->setItemDelegate(m_bookmarksDelegate);
@@ -147,44 +151,31 @@ QGraphicsWidget *WebBrowser::graphicsWidget()
     m_removeBookmarkAction = new QAction(KIcon("list-remove"), QString(), this);
     m_organizeBookmarks = addTool("bookmarks-organize", m_statusbarLayout);
 
+    m_bookmarksViewAnimation = Plasma::Animator::create(Plasma::Animator::FadeAnimation, this);
+    m_bookmarksViewAnimation->setTargetWidget(m_bookmarksView);
+    connect(m_bookmarksViewAnimation, SIGNAL(finished()), this, SLOT(bookmarksAnimationFinished()));
 
-    m_progress = new Plasma::Meter(this);
-    m_progress->setMeterType(Plasma::Meter::BarMeterHorizontal);
-    m_progress->setMinimum(0);
-    m_progress->setMaximum(100);
-    m_statusbarLayout->addItem(m_progress);
     m_stop = addTool("process-stop", m_statusbarLayout);
 
+    QGraphicsWidget *spacer = new QGraphicsWidget(this);
+    spacer->setMaximumWidth(INT_MAX);
+    spacer->setMaximumHeight(0);
+    spacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+    m_statusbarLayout->addItem(spacer);
+    
     m_zoom = new Plasma::Slider(this);
     m_zoom->setMaximum(100);
     m_zoom->setMinimum(0);
     m_zoom->setValue(50);
     m_zoom->setOrientation(Qt::Horizontal);
     m_zoom->hide();
-    m_zoom->setMaximumWidth(0);
+    m_zoom->setMaximumWidth(200);
     m_statusbarLayout->addItem(m_zoom);
 
-    if (!m_url.isValid()) {
-        m_url = KUrl(cg.readEntry("Url", "http://www.kde.org"));
-        m_verticalScrollValue = cg.readEntry("VerticalScrollValue", 0);
-        m_horizontalScrollValue = cg.readEntry("HorizontalScrollValue", 0);
-        int value = cg.readEntry("Zoom", 1);
-        m_zoom->setValue(value);
-        m_browser->mainFrame()->setZoomFactor((qreal)0.2 + ((qreal)value/(qreal)50));
-    }
     connect(m_zoom, SIGNAL(valueChanged(int)), this, SLOT(zoom(int)));
     m_browser->setUrl(m_url);
     m_browser->update();
 
-    m_autoRefresh = cg.readEntry("autoRefresh", false);
-    m_autoRefreshInterval = qMax(2, cg.readEntry("autoRefreshInterval", 5));
-
-    if (m_autoRefresh) {
-        m_autoRefreshTimer = new QTimer(this);
-        m_autoRefreshTimer->start(m_autoRefreshInterval*60*1000);
-        connect(m_autoRefreshTimer, SIGNAL(timeout()), this, SLOT(reload()));
-    }
-
     connect(m_back->action(), SIGNAL(triggered()), this, SLOT(back()));
     connect(m_forward->action(), SIGNAL(triggered()), this, SLOT(forward()));
     connect(m_reloadAction, SIGNAL(triggered()), this, SLOT(reload()));
@@ -196,7 +187,7 @@ QGraphicsWidget *WebBrowser::graphicsWidget()
     connect(m_historyCombo, SIGNAL(activated(const QString&)), this, SLOT(comboTextChanged(const QString&)));
     connect(m_browser->page()->mainFrame(), SIGNAL(urlChanged(const QUrl &)), this, SLOT(urlChanged(const QUrl &)));
     connect(m_browser, SIGNAL(loadProgress(int)), this, SLOT(loadProgress(int)));
-
+    
     connect(m_addBookmarkAction, SIGNAL(triggered()), this, SLOT(addBookmark()));
     connect(m_removeBookmarkAction, SIGNAL(triggered()), this, SLOT(removeBookmark()));
     connect(m_organizeBookmarks->action(), SIGNAL(triggered()), this, SLOT(bookmarksToggle()));
@@ -206,21 +197,23 @@ QGraphicsWidget *WebBrowser::graphicsWidget()
     m_completion = new KCompletion();
     m_nativeHistoryCombo->setCompletionObject(m_completion);
 
-    QStringList list = cg.readEntry("History list", QStringList());
-    m_nativeHistoryCombo->setHistoryItems(list);
-
     m_graphicsWidget = new QGraphicsWidget(this);
     m_graphicsWidget->setLayout(m_layout);
 
     m_back->setEnabled(m_browser->page()->history()->canGoBack());
     m_forward->setEnabled(m_browser->page()->history()->canGoForward());
 
+    configChanged();
 
+    connect(this, SIGNAL(messageButtonPressed(const MessageButton)), this, SLOT(removeBookmarkMessageButtonPressed(const MessageButton)));
+    
     return m_graphicsWidget;
 }
 
 WebBrowser::~WebBrowser()
 {
+    KConfigGroup cg = config();
+    saveState(cg);
     delete m_completion;
     delete m_bookmarkModel;
 }
@@ -247,6 +240,39 @@ void WebBrowser::bookmarksModelInit()
     fillGroup(0, m_bookmarkManager->root());
 }
 
+void WebBrowser::configChanged()
+{
+    KConfigGroup cg = config();
+    
+    m_browser->setDragToScroll(cg.readEntry("DragToScroll", false));
+    
+    if (!m_url.isValid()) {
+        m_url = KUrl(cg.readEntry("Url", "http://www.kde.org"));
+        m_verticalScrollValue = cg.readEntry("VerticalScrollValue", 0);
+        m_horizontalScrollValue = cg.readEntry("HorizontalScrollValue", 0);
+        int value = cg.readEntry("Zoom", 50);
+        m_zoom->setValue(value);
+        qreal zoomFactor = qMax((qreal)0.2, ((qreal)value/(qreal)50));
+        if ((zoomFactor > 0.95) && (zoomFactor < 1.05)){
+            zoomFactor = 1;
+        }
+        m_browser->setZoomFactor(zoomFactor);
+        m_browser->setUrl(m_url);
+    }
+    
+    m_autoRefresh = cg.readEntry("autoRefresh", false);
+    m_autoRefreshInterval = qMax(2, cg.readEntry("autoRefreshInterval", 5));
+    
+    if (m_autoRefresh) {
+        m_autoRefreshTimer = new QTimer(this);
+        m_autoRefreshTimer->start(m_autoRefreshInterval*60*1000);
+        connect(m_autoRefreshTimer, SIGNAL(timeout()), this, SLOT(reload()));
+    }
+        
+    QStringList list = cg.readEntry("History list", QStringList());
+    m_nativeHistoryCombo->setHistoryItems(list);
+}
+
 void WebBrowser::fillGroup(BookmarkItem *parentItem, const KBookmarkGroup &group)
 {
     KBookmark it = group.first();
@@ -386,18 +412,8 @@ void WebBrowser::removeBookmark(const QModelIndex &index)
         KBookmark bookmark = item->bookmark();
 
         const QString text(i18nc("@info", "Do you really want to remove the bookmark to %1?", bookmark.url().host()));
-        const bool del = KMessageBox::warningContinueCancel(0,
-                                                            text,
-                                                            QString(),
-                                                            KGuiItem(i18nc("@action:button", "Delete Bookmark"))
-                                                            ) == KMessageBox::Continue;
-
-        if (!del) {
-            return;
-        }
-
-        bookmark.parentGroup().deleteBookmark(bookmark);
-        m_bookmarkManager->save();
+        showMessage(KIcon("dialog-warning"), text, Plasma::ButtonYes | Plasma::ButtonNo);
+        return;
     }
 
     if (item && item->parent()) {
@@ -408,6 +424,34 @@ void WebBrowser::removeBookmark(const QModelIndex &index)
 
 }
 
+void WebBrowser::removeBookmarkMessageButtonPressed(const Plasma::MessageButton button)
+{  
+    if (button == Plasma::ButtonNo){
+        return;
+    }
+    
+    const QModelIndexList list = m_bookmarkModel->match(m_bookmarkModel->index(0,0), BookmarkItem::UrlRole, m_url.prettyUrl());
+
+    if (!list.isEmpty()) {
+        const QModelIndex &index = list.first();
+        BookmarkItem *item = dynamic_cast<BookmarkItem *>(m_bookmarkModel->itemFromIndex(index));
+        
+        if (item) {
+            KBookmark bookmark = item->bookmark();
+            
+            bookmark.parentGroup().deleteBookmark(bookmark);
+            m_bookmarkManager->save();
+        }
+        
+        if (item && item->parent()) {
+            item->parent()->removeRow(index.row());
+        } else {
+            m_bookmarkModel->removeRow(index.row());
+        }
+    }
+    
+    m_addBookmark->setAction(m_addBookmarkAction);
+}
 void WebBrowser::removeBookmark()
 {
     const QModelIndexList list = m_bookmarkModel->match(m_bookmarkModel->index(0,0), BookmarkItem::UrlRole, m_url.prettyUrl());
@@ -415,22 +459,28 @@ void WebBrowser::removeBookmark()
     if (!list.isEmpty()) {
         removeBookmark(list.first());
     }
-
-    m_addBookmark->setAction(m_addBookmarkAction);
 }
 
 void WebBrowser::bookmarksToggle()
 {
     if (m_bookmarksView->isVisible()) {
-        m_bookmarksView->hide();
-        m_bookmarksView->setMaximumHeight(0);
-        m_browser->show();
-        m_browser->setMaximumHeight(INT_MAX);
+        m_bookmarksViewAnimation->setProperty("startOpacity", 1);
+        m_bookmarksViewAnimation->setProperty("targetOpacity", 0);
+        m_bookmarksViewAnimation->start();
     } else {
         m_bookmarksView->show();
-        m_bookmarksView->setMaximumHeight(INT_MAX);
-        m_browser->hide();
-        m_browser->setMaximumHeight(0);
+        m_bookmarksView->setOpacity(0);
+        updateOverlaysGeometry();
+        m_bookmarksViewAnimation->setProperty("startOpacity", 0);
+        m_bookmarksViewAnimation->setProperty("targetOpacity", 1);
+        m_bookmarksViewAnimation->start();
+    }
+}
+
+void WebBrowser::bookmarksAnimationFinished()
+{
+    if (qFuzzyCompare(m_bookmarksView->opacity() + 1, 1)){
+        m_bookmarksView->hide();
     }
 }
 
@@ -451,31 +501,27 @@ void WebBrowser::bookmarkClicked(const QModelIndex &index)
 void WebBrowser::zoom(int value)
 {
     config().writeEntry("Zoom", value);
-    m_browser->mainFrame()->setZoomFactor((qreal)0.2 + ((qreal)value/(qreal)50));
+    m_browser->setZoomFactor((qreal)0.2 + ((qreal)value/(qreal)50));
 }
 
 void WebBrowser::loadProgress(int progress)
 {
-    m_progress->setValue(progress);
-    m_progress->update();
+    m_historyCombo->setProgressValue(progress);
 
     if (progress == 100) {
-        m_progress->setMaximumWidth(0);
-        m_progress->hide();
+        m_historyCombo->setDisplayProgress(false);
         m_stop->hide();
         m_stop->setMaximumWidth(0);
         m_zoom->show();
-        m_zoom->setMaximumWidth(INT_MAX);
         m_statusbarLayout->invalidate();
 
         m_browser->page()->mainFrame()->setScrollBarValue(Qt::Vertical, m_verticalScrollValue);
         m_browser->page()->mainFrame()->setScrollBarValue(Qt::Horizontal, m_horizontalScrollValue);
+
     } else {
-        m_progress->show();
+        m_historyCombo->setDisplayProgress(true);
         m_stop->show();
         m_stop->setMaximumWidth(INT_MAX);
-        m_progress->setMaximumWidth(INT_MAX);
-        m_zoom->setMaximumWidth(0);
         m_zoom->hide();
         m_statusbarLayout->invalidate();
     }
@@ -522,4 +568,103 @@ void WebBrowser::configAccepted()
     emit configNeedsSaving();
 }
 
+void WebBrowser::constraintsEvent(Plasma::Constraints constraints)
+{
+    if (constraints & Plasma::SizeConstraint){
+        updateOverlaysGeometry();
+    }
+}
+
+void WebBrowser::updateOverlaysGeometry()
+{
+    QRect overlayGeometry(m_browser->pos().x() + contentsRect().x(),
+                          m_browser->pos().y() + contentsRect().y(),
+                          m_browser->geometry().width(),
+                          m_browser->geometry().height());
+
+    if (m_bookmarksView->isVisible()) {
+      m_bookmarksView->setGeometry(overlayGeometry);
+    }
+      
+    if (m_webOverlay){
+      m_webOverlay->setGeometry(overlayGeometry);
+    }
+}
+
+void WebBrowser::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
+{
+    Q_UNUSED(option)
+    Q_UNUSED(contentsRect)
+  
+    p->save();
+    p->setBrush(QApplication::palette().window());
+    p->setRenderHint(QPainter::Antialiasing);
+    p->setPen(Qt::NoPen);
+    p->drawRoundedRect(m_browser->pos().x() + contentsRect.x() - 2,
+                       m_browser->pos().y() + contentsRect.y() - 2,
+                       m_browser->geometry().width() + 4,
+                       m_browser->geometry().height() + 4,
+                       2, 2);
+    p->restore();
+}
+
+void WebBrowser::closeWebViewOverlay()
+{
+    if (m_webOverlay){
+      m_webOverlay->deleteLater();
+      m_webOverlay = 0;
+    }
+}
+
+QWebPage *WebBrowser::createWindow(QWebPage::WebWindowType type)
+{
+    Q_UNUSED(type)
+  
+    if (!m_webOverlay){
+        m_webOverlay = new WebViewOverlay(this);
+        updateOverlaysGeometry();
+        m_webOverlay->setZValue(999);
+        connect(m_webOverlay, SIGNAL(closeRequested()), this, SLOT(closeWebViewOverlay()));
+    }
+
+    return m_webOverlay->page();
+}
+
+//
+// Wallet managment
+//
+
+void WebBrowser::saveFormDataRequested(const QString &uid, const QUrl &url)
+{
+    BrowserMessageBox *messageBox = new BrowserMessageBox(this, i18n("Do you want to store this password for %1?", url.host()));
+    messageBox->okButton()->setText(i18n("Store"));
+    messageBox->okButton()->setIcon(KIcon("document-save"));
+    messageBox->cancelButton()->setText(i18n("Do not store this time"));
+    messageBox->cancelButton()->setIcon(KIcon("dialog-cancel"));
+    m_layout->insertItem(1, messageBox);
+    walletRequests.insert(messageBox, uid);
+    connect(messageBox, SIGNAL(okClicked()), this, SLOT(acceptWalletRequest()));
+    connect(messageBox, SIGNAL(cancelClicked()), this, SLOT(rejectWalletRequest()));
+}
+
+void WebBrowser::acceptWalletRequest()
+{
+    static_cast<KWebPage *>(m_browser->page())->wallet()->acceptSaveFormDataRequest(
+                            walletRequests[static_cast<BrowserMessageBox *>(QObject::sender())]);
+    QObject::sender()->deleteLater();
+}
+
+void WebBrowser::rejectWalletRequest()
+{
+    static_cast<KWebPage *>(m_browser->page())->wallet()->rejectSaveFormDataRequest(
+                           walletRequests[static_cast<BrowserMessageBox *>(QObject::sender())]);
+    QObject::sender()->deleteLater();
+}
+
+//
+// End of wallet managment
+//
+
+K_EXPORT_PLASMA_APPLET(webbrowser, WebBrowser)
+
 #include "webbrowser.moc"
