@@ -40,8 +40,9 @@
 #include <kephal/screens.h>
 
 #include <Plasma/Containment>
-#include <Plasma/Theme>
 #include <Plasma/Dialog>
+#include <Plasma/Theme>
+#include <Plasma/Wallpaper>
 #include <Plasma/WindowEffects>
 
 #include "netcorona.h"
@@ -204,6 +205,109 @@ private:
 };
 
 
+class ShadowWindow : public QWidget
+{
+public:
+    ShadowWindow(NetView *panel)
+       : QWidget(0),
+         m_panel(panel),
+         m_valid(false)
+    {
+        setAttribute(Qt::WA_TranslucentBackground);
+        setAttribute(Qt::WA_NoSystemBackground, false);
+        setAutoFillBackground(false);
+#ifdef Q_WS_X11
+        QRegion region(QRect(0,0,1,1));
+        XShapeCombineRegion(QX11Info::display(), winId(), ShapeInput, 0, 0,
+                            region.handle(), ShapeSet);
+#endif
+
+        m_shadow = new Plasma::FrameSvg(this);
+    }
+
+    void setSvg(const QString &path)
+    {
+        m_shadow->setImagePath(path);
+
+        if (!m_shadow->hasElementPrefix("shadow")) {
+            hide();
+            m_valid = false;
+        } else {
+            m_valid = true;
+        }
+
+        m_shadow->setElementPrefix("shadow");
+
+        adjustMargins(geometry());
+    }
+
+    bool isValid() const
+    {
+        return m_valid;
+    }
+
+    void adjustMargins(const QRect &geo)
+    {
+        QRect screenRect = Kephal::ScreenUtils::screenGeometry(m_panel->screen());
+
+        Plasma::FrameSvg::EnabledBorders enabledBorders = Plasma::FrameSvg::AllBorders;
+
+        if (geo.left() <= screenRect.left()) {
+            enabledBorders ^= Plasma::FrameSvg::LeftBorder;
+        }
+        if (geo.top() <= screenRect.top()) {
+            enabledBorders ^= Plasma::FrameSvg::TopBorder;
+        }
+        if (geo.bottom() >= screenRect.bottom()) {
+            enabledBorders ^= Plasma::FrameSvg::BottomBorder;
+        }
+        if (geo.right() >= screenRect.right()) {
+            enabledBorders ^= Plasma::FrameSvg::RightBorder;
+        }
+
+        m_shadow->setEnabledBorders(enabledBorders);
+
+        qreal left, top, right, bottom;
+
+        m_shadow->getMargins(left, top, right, bottom);
+        setContentsMargins(left, top, right, bottom);
+    }
+
+protected:
+    bool event(QEvent *event)
+    {
+        Q_UNUSED(event)
+
+        if (event->type() == QEvent::Paint) {
+            QPainter p(this);
+            p.setCompositionMode(QPainter::CompositionMode_Source);
+            p.fillRect(rect(), Qt::transparent);
+        }
+        return QWidget::event(event);
+    }
+
+    void resizeEvent(QResizeEvent *event)
+    {
+        m_shadow->resizeFrame(event->size());
+
+        adjustMargins(geometry());
+    }
+
+    void paintEvent(QPaintEvent* e)
+    {
+        Q_UNUSED(e)
+
+        QPainter p(this);
+        //p.setCompositionMode(QPainter::CompositionMode_Source);
+        m_shadow->paintFrame(&p);
+    }
+
+private:
+    Plasma::FrameSvg *m_shadow;
+    NetView *m_panel;
+    bool m_valid;
+};
+
 PlasmaApp::PlasmaApp()
     : KUniqueApplication(),
       m_corona(0),
@@ -215,9 +319,11 @@ PlasmaApp::PlasmaApp()
       m_mainView(0),
       m_isDesktop(false),
       m_autoHideControlBar(true),
-      m_raiseTimer(new QTimer(this)),
-      m_unHideTimer(0)
+      m_unHideTimer(0),
+      m_shadowWindow(0),
+      m_startupSuspendWaitCount(0)
 {
+    PlasmaApp::suspendStartup(true);
     KGlobal::locale()->insertCatalog("libplasma");
     KGlobal::locale()->insertCatalog("plasmagenericshell");
 
@@ -225,7 +331,6 @@ PlasmaApp::PlasmaApp()
     KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
     bool isDesktop = args->isSet("desktop");
     if (isDesktop) {
-        notifyStartup(false);
         KCrash::setFlags(KCrash::AutoRestart);
     }
 
@@ -247,13 +352,13 @@ PlasmaApp::PlasmaApp()
     Plasma::Theme::defaultTheme()->setFont(cg.readEntry("desktopFont", font()));
 
     m_mainView = new NetView(0, NetView::mainViewId(), 0);
+    m_mainView->hide();
 
-    m_raiseTimer->setSingleShot(true);
-    connect(m_raiseTimer, SIGNAL(timeout()), this, SLOT(raiseMainView()));
-	
     connect(m_mainView, SIGNAL(containmentActivated()), this, SLOT(mainContainmentActivated()));
     connect(KWindowSystem::self(), SIGNAL(workAreaChanged()), this, SLOT(positionPanel()));
-    m_mainView->installEventFilter(this);
+
+    bool useGL = args->isSet("opengl");
+    m_mainView->setUseGL(useGL);
 
     connect(KWindowSystem::self(), SIGNAL(activeWindowChanged(WId)), this, SLOT(controlBarVisibilityUpdate()));
 
@@ -276,19 +381,16 @@ PlasmaApp::PlasmaApp()
         }
     }
 
-    setIsDesktop(isDesktop);
     m_mainView->setFixedSize(width, height);
     m_mainView->move(0,0);
+    setIsDesktop(isDesktop);
 
     // this line initializes the corona.
     corona();
     //setIsDesktop(isDesktop);
     reserveStruts();
 
-    if (isDesktop) {
-        notifyStartup(true);
-    }
-
+    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(checkShadow()));
     connect(this, SIGNAL(aboutToQuit()), this, SLOT(cleanup()));
 }
 
@@ -338,6 +440,9 @@ void PlasmaApp::positionPanel()
     }
 
     QRect screenRect = Kephal::ScreenUtils::screenGeometry(m_controlBar->screen());
+    if (!m_isDesktop) {
+        screenRect = m_mainView->geometry();
+    }
 
     //move
     controlBarMoved(m_controlBar);
@@ -349,55 +454,54 @@ void PlasmaApp::positionPanel()
     }
 
 
-    int left = 0;
-    int right = 0;
-    int top = 0;
-    int bottom = 0;
-
-    switch (m_controlBar->location()) {
-    case Plasma::LeftEdge:
-        left = m_controlBar->width();
-        break;
-    case Plasma::RightEdge:
-        right = m_controlBar->width();
-        break;
-    case Plasma::TopEdge:
-        top = m_controlBar->height();
-        break;
-    case Plasma::BottomEdge:
-        bottom = m_controlBar->height();
-        break;
-    default:
-        break;
-    }
-
     m_controlBar->containment()->setMaximumSize(m_controlBar->size());
     m_controlBar->containment()->setMinimumSize(m_controlBar->size());
 
-    if (m_widgetExplorerView) {
-        top += m_widgetExplorerView->size().height();
-    }
-
-    foreach (Plasma::Containment *containment, m_corona->containments()) {
-        if (containment->location() == Plasma::Desktop ||
-            containment->location() == Plasma::Floating) {
-            containment->setContentsMargins(left, top, right, bottom);
-        }
-    }
-
     if (m_autoHideControlBar && m_controlBar->isVisible()) {
         destroyUnHideTrigger();
         createUnhideTrigger();
     }
+
+    checkShadow();
+
+    emit controlBarChanged();
+}
+
+void PlasmaApp::checkShadow()
+{
+    if (!m_controlBar) {
+        return;
+    }
+
+    if (KWindowSystem::compositingActive() && m_controlBar->containment()->property("shadowPath").isValid()) {
+        if (!m_shadowWindow) {
+            m_shadowWindow = new ShadowWindow(m_controlBar);
+            KWindowSystem::setOnAllDesktops(m_controlBar->winId(), true);
+        }
+        KWindowSystem::setType(m_shadowWindow->winId(), NET::Dock);
+        KWindowSystem::setState(m_shadowWindow->winId(), NET::KeepBelow);
+        KWindowSystem::setOnAllDesktops(m_shadowWindow->winId(), true);
+        m_shadowWindow->setSvg(m_controlBar->containment()->property("shadowPath").toString());
+        int left, right, top, bottom;
+        m_shadowWindow->adjustMargins(m_controlBar->geometry());
+        m_shadowWindow->getContentsMargins(&left, &top, &right, &bottom);
+        m_shadowWindow->setMinimumSize(-1, -1);
+        m_shadowWindow->setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
+        m_shadowWindow->setGeometry(m_controlBar->geometry().adjusted(-left, -top, right, bottom));
+        m_shadowWindow->setFixedSize(m_shadowWindow->size());
+        if (m_shadowWindow->isValid()) {
+            m_shadowWindow->show();
+        }
+    } else {
+        m_shadowWindow->deleteLater();
+        m_shadowWindow = 0;
+    }
 }
 
 void PlasmaApp::mainContainmentActivated()
 {
     m_mainView->setWindowTitle(m_mainView->containment()->activity());
 
-    if (!m_isDesktop) {
-        return;
-    }
 
     const WId id = m_mainView->effectiveWinId();
 
@@ -405,10 +509,13 @@ void PlasmaApp::mainContainmentActivated()
     KWindowSystem::raiseWindow(id);
 
     if (activeWindow) {
-        KWindowSystem::clearState(m_mainView->winId(), NET::KeepBelow);
         KWindowSystem::raiseWindow(activeWindow->effectiveWinId());
         m_mainView->activateWindow();
         activeWindow->setFocus();
+        if (m_shadowWindow) {
+            KWindowSystem::clearState(m_shadowWindow->winId(), NET::KeepBelow);
+            KWindowSystem::setState(m_shadowWindow->winId(), NET::KeepAbove);
+        }
     } else {
         m_mainView->activateWindow();
     }
@@ -419,13 +526,13 @@ void PlasmaApp::setIsDesktop(bool isDesktop)
     m_isDesktop = isDesktop;
 
     if (isDesktop) {
+        KWindowSystem::setType(m_mainView->winId(), NET::Normal);
         m_mainView->setWindowFlags(m_mainView->windowFlags() | Qt::FramelessWindowHint);
         KWindowSystem::setOnAllDesktops(m_mainView->winId(), true);
         if (m_controlBar) {
             KWindowSystem::setOnAllDesktops(m_controlBar->winId(), true);
         }
         m_mainView->show();
-        KWindowSystem::setType(m_mainView->winId(), NET::Normal);
     } else {
         m_mainView->setWindowFlags(m_mainView->windowFlags() & ~Qt::FramelessWindowHint);
         KWindowSystem::setOnAllDesktops(m_mainView->winId(), false);
@@ -457,7 +564,12 @@ void PlasmaApp::adjustSize(Kephal::Screen *screen)
 
 void PlasmaApp::reserveStruts()
 {
-    if (!m_controlBar || !isDesktop()) {
+    if (!m_controlBar || m_autoHideControlBar || !isDesktop()) {
+        KWindowSystem::setExtendedStrut(m_controlBar->winId(),
+                                    0, 0, 0,
+                                    0, 0, 0,
+                                    0, 0, 0,
+                                    0, 0, 0);
         return;
     }
 
@@ -509,6 +621,11 @@ NetView *PlasmaApp::mainView() const
     return m_mainView;
 }
 
+QWidget *PlasmaApp::widgetExplorer() const
+{
+    return m_widgetExplorerView;
+}
+
 Plasma::Corona* PlasmaApp::corona()
 {
     if (!m_corona) {
@@ -517,34 +634,63 @@ Plasma::Corona* PlasmaApp::corona()
                 this, SLOT(createView(Plasma::Containment*)));
         connect(m_corona, SIGNAL(configSynced()), this, SLOT(syncConfig()));
 
-
-        m_corona->setItemIndexMethod(QGraphicsScene::NoIndex);
-        m_corona->initializeLayout();
-
-        m_mainView->show();
-
         connect(m_corona, SIGNAL(screenOwnerChanged(int,int,Plasma::Containment*)),
                 m_mainView, SLOT(screenOwnerChanged(int,int,Plasma::Containment*)));
 
+        m_corona->setItemIndexMethod(QGraphicsScene::NoIndex);
+        m_corona->initializeLayout();
+        m_corona->processUpdateScripts();
+
+        m_mainView->show();
     }
 
+    foreach (Plasma::Containment *containment, m_corona->containments()) {
+        if (containment->screen() != -1 && containment->wallpaper()) {
+            ++m_startupSuspendWaitCount;
+            connect(containment->wallpaper(), SIGNAL(update(QRectF)), this, SLOT(wallpaperCheckedIn()));
+        }
+    }
+
+    QTimer::singleShot(5000, this, SLOT(wallpaperCheckInTimeout()));
+
     return m_corona;
 }
 
+void PlasmaApp::wallpaperCheckInTimeout()
+{
+    if (m_startupSuspendWaitCount > 0) {
+        m_startupSuspendWaitCount = 0;
+        suspendStartup(false);
+    }
+}
+
+void PlasmaApp::wallpaperCheckedIn()
+{
+    if (m_startupSuspendWaitCount < 1) {
+        return;
+    }
+
+    --m_startupSuspendWaitCount;
+    if (m_startupSuspendWaitCount < 1) {
+        m_startupSuspendWaitCount = 0;
+        suspendStartup(false);
+    }
+}
+
 bool PlasmaApp::hasComposite()
 {
     return KWindowSystem::compositingActive();
 }
 
-void PlasmaApp::notifyStartup(bool completed)
+void PlasmaApp::suspendStartup(bool suspend)
 {
     org::kde::KSMServerInterface ksmserver("org.kde.ksmserver", "/KSMServer", QDBusConnection::sessionBus());
 
-    const QString startupID("workspace desktop");
-    if (completed) {
-        ksmserver.resumeStartup(startupID);
-    } else {
+    const QString startupID("netbook desktop");
+    if (suspend) {
         ksmserver.suspendStartup(startupID);
+    } else {
+        ksmserver.resumeStartup(startupID);
     }
 }
 
@@ -598,6 +744,7 @@ void PlasmaApp::createView(Plasma::Containment *containment)
         }
 
         m_controlBar->setContainment(containment);
+        positionPanel();
         setControlBarVisible(true);
         containment->setMaximumSize(m_controlBar->size());
         containment->setMinimumSize(m_controlBar->size());
@@ -606,6 +753,8 @@ void PlasmaApp::createView(Plasma::Containment *containment)
         m_autoHideControlBar = m_controlBar->config().readEntry("panelAutoHide", true);
 
         setAutoHideControlBar(m_autoHideControlBar);
+        emit controlBarChanged();
+        setControlBarVisible(true);
     } else {
         containment->setScreen(-1);
     }
@@ -622,11 +771,16 @@ void PlasmaApp::closeWidgetExplorer()
 
 void PlasmaApp::updateToolBoxVisibility(bool visible)
 {
+    bool hadToolBoxOpen = false;
+
     foreach (Plasma::Containment *cont, m_corona->containments()) {
+        if (cont->isToolBoxOpen()) {
+            hadToolBoxOpen = true;
+        }
          cont->setToolBoxOpen(visible);
     }
 
-    if (!visible) {
+    if (!visible && hadToolBoxOpen) {
         closeWidgetExplorer();
     }
 }
@@ -754,6 +908,7 @@ void PlasmaApp::showWidgetExplorer(Plasma::Containment *containment)
             m_widgetExplorerView->resize(m_mainView->size().width(), KIconLoader::SizeEnormous);
             m_widgetExplorerView->move(0,0);
         }
+
     }
 
     if (!m_widgetExplorer) {
@@ -768,18 +923,27 @@ void PlasmaApp::showWidgetExplorer(Plasma::Containment *containment)
         m_widgetExplorerView->setGraphicsWidget(m_widgetExplorer);
 
         m_widgetExplorer->setIconSize(KIconLoader::SizeLarge);
-        m_widgetExplorer->resize(m_widgetExplorerView->size());
-        m_widgetExplorerView->setFixedSize(viewSize);
         m_widgetExplorerView->installEventFilter(this);
     }
 
-    m_widgetExplorer->setOrientation(Qt::Horizontal);
-    positionPanel();
+    const bool vert = m_controlBar->location() == Plasma::LeftEdge ||
+                      m_controlBar->location() == Plasma::RightEdge;
+    m_widgetExplorer->setOrientation(vert ? Qt::Vertical : Qt::Horizontal);
 
+    if (m_widgetExplorer->orientation() == Qt::Horizontal) {
+        m_widgetExplorer->setMinimumWidth(m_mainView->size().width());
+        m_widgetExplorer->setMinimumHeight(-1);
+    } else {
+        m_widgetExplorer->setMinimumWidth(-1);
+        m_widgetExplorer->setMinimumHeight(m_mainView->size().height());
+    }
+
+    positionPanel();
 
     m_widgetExplorer->show();
     Plasma::WindowEffects::slideWindow(m_widgetExplorerView, m_controlBar->location());
     m_widgetExplorerView->show();
+    emit controlBarChanged();
 }
 
 void PlasmaApp::widgetExplorerDestroyed()
@@ -787,7 +951,9 @@ void PlasmaApp::widgetExplorerDestroyed()
     m_widgetExplorer = 0;
     m_widgetExplorerView = 0;
     positionPanel();
-    m_mainView->containment()->setToolBoxOpen(false);
+    if (m_mainView->containment()) {
+        m_mainView->containment()->setToolBoxOpen(false);
+    }
 }
 
 
@@ -815,30 +981,34 @@ void PlasmaApp::configureContainment(Plasma::Containment *containment)
     KWindowSystem::activateWindow(configDialog->winId());
 }
 
+bool PlasmaApp::mainViewOnTop() const
+{
+    bool onTop = false;
+
+    QSet<WId> ownWindows;
+    foreach (QWidget *widget, QApplication::topLevelWidgets()) {
+        ownWindows.insert(widget->winId());
+    }
+
+    //search if the main view is actually one of the widgets on top, show the panel only in this case
+    QList<WId> windows = KWindowSystem::stackingOrder();
+    for (int i = windows.size() - 1; i >= 0; --i) {
+        WId window = windows.at(i);
+
+        if (window == m_mainView->winId()) {
+            onTop = true;
+            break;
+        } else if (!ownWindows.contains(window)) {
+            break;
+        }
+    }
+
+    return onTop;
+}
 
 bool PlasmaApp::eventFilter(QObject * watched, QEvent *event)
 {
-    if (watched == m_mainView && event->type() == QEvent::WindowActivate) {
-        destroyUnHideTrigger();
-        if (m_controlBar) {
-            Plasma::WindowEffects::slideWindow(m_controlBar, m_controlBar->location());
-            setControlBarVisible(true);
-            m_raiseTimer->start(100);
-            reserveStruts();
-        }
-    } else if ((watched == m_mainView &&
-                event->type() == QEvent::WindowDeactivate &&
-                !hasForegroundWindows()) ||
-               (watched == m_controlBar &&
-                event->type() == QEvent::Leave &&
-                !hasForegroundWindows())) {
-kWarning()<<event;
-        //delayed hide
-        if (m_unHideTimer) {
-            m_unHideTimer->start(400);
-            reserveStruts();
-        }
-    } else if (watched == m_widgetExplorerView && event->type() == QEvent::KeyPress) {
+    if (watched == m_widgetExplorerView && event->type() == QEvent::KeyPress) {
         QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
         if (keyEvent->key() == Qt::Key_Escape) {
             closeWidgetExplorer();
@@ -876,9 +1046,11 @@ bool PlasmaApp::x11EventFilter(XEvent *event)
         }
     } else if ((event->xany.send_event != True && event->type == FocusOut)) {
         QTimer::singleShot(100, this, SLOT(lowerMainView()));
-    } else if ((event->xany.send_event != True && event->type == ButtonPress)) {
-        m_raiseTimer->stop();
+    } else if (m_controlBar && m_autoHideControlBar && m_controlBar->isVisible() &&
+        (event->xany.send_event != True && event->type == LeaveNotify)) {
+        m_unHideTimer->start(200);
     }
+
     return KUniqueApplication::x11EventFilter(event);
 }
 
@@ -891,15 +1063,10 @@ void PlasmaApp::lowerMainView()
 {
     if (m_isDesktop && !hasForegroundWindows()) {
         KWindowSystem::lowerWindow(m_mainView->winId());
-        KWindowSystem::setState(m_mainView->winId(), NET::KeepBelow);
     }
-}
-
-void PlasmaApp::raiseMainView()
-{
-    if (m_isDesktop) {
-        KWindowSystem::clearState(m_mainView->winId(), NET::KeepBelow);
-        KWindowSystem::raiseWindow(m_mainView->winId());
+    if (m_shadowWindow) {
+        KWindowSystem::clearState(m_shadowWindow->winId(), NET::KeepAbove);
+        KWindowSystem::setState(m_shadowWindow->winId(), NET::KeepBelow);
     }
 }
 
@@ -911,23 +1078,58 @@ void PlasmaApp::controlBarVisibilityUpdate()
 
     if (!m_autoHideControlBar) {
         setControlBarVisible(true);
+
+        if (m_shadowWindow && m_shadowWindow->isValid()) {
+            Plasma::WindowEffects::slideWindow(m_shadowWindow, m_controlBar->location());
+            m_shadowWindow->show();
+            if (hasForegroundWindows()) {
+                KWindowSystem::clearState(m_shadowWindow->winId(), NET::KeepBelow);
+                KWindowSystem::setState(m_shadowWindow->winId(), NET::KeepAbove);
+            } else {
+                KWindowSystem::clearState(m_shadowWindow->winId(), NET::KeepAbove);
+                KWindowSystem::setState(m_shadowWindow->winId(), NET::KeepBelow);
+            }
+            KWindowSystem::setOnAllDesktops(m_shadowWindow->winId(), true);
+        }
+
         return;
     } else if (m_autoHideControlBar && hasForegroundWindows() && m_controlBar->isVisible()) {
         return;
     }
 
+    if (sender() != m_unHideTimer) {
+        m_unHideTimer->start(200);
+        return;
+    }
+
     //would be nice to avoid this
     QPoint cursorPos = QCursor::pos();
 
-    if (m_triggerZone.adjusted(-1, -1, 1, 1).contains(cursorPos)) { 
+    if (m_triggerZone.adjusted(-1, -1, 1, 1).contains(cursorPos) || hasForegroundWindows()) {
         if (!m_controlBar->isVisible()) {
             destroyUnHideTrigger();
             Plasma::WindowEffects::slideWindow(m_controlBar, m_controlBar->location());
             setControlBarVisible(true);
         }
-    } else if (!m_controlBar->geometry().contains(cursorPos)) {
+
+        if (m_shadowWindow && m_shadowWindow->isValid()) {
+            Plasma::WindowEffects::slideWindow(m_shadowWindow, m_controlBar->location());
+            if (hasForegroundWindows()) {
+                KWindowSystem::clearState(m_shadowWindow->winId(), NET::KeepBelow);
+                KWindowSystem::setState(m_shadowWindow->winId(), NET::KeepAbove);
+            }
+            m_shadowWindow->show();
+            KWindowSystem::setOnAllDesktops(m_shadowWindow->winId(), true);
+        }
+    } else if (!m_controlBar->geometry().contains(cursorPos) && !mainViewOnTop() && !hasForegroundWindows()) {
         Plasma::WindowEffects::slideWindow(m_controlBar, m_controlBar->location());
         m_controlBar->hide();
+
+        if (m_shadowWindow) {
+            Plasma::WindowEffects::slideWindow(m_shadowWindow, m_controlBar->location());
+            m_shadowWindow->hide();
+        }
+
         createUnhideTrigger();
     }
 }
@@ -951,17 +1153,31 @@ void PlasmaApp::setControlBarVisible(bool visible)
     if (visible) {
         destroyUnHideTrigger();
         Plasma::WindowEffects::slideWindow(m_controlBar, m_controlBar->location());
-        m_controlBar->show();
         KWindowSystem::setOnAllDesktops(m_controlBar->effectiveWinId(), m_isDesktop);
         m_controlBar->setWindowFlags(m_mainView->windowFlags() | Qt::FramelessWindowHint);
         m_controlBar->setFrameShape(QFrame::NoFrame);
+        m_controlBar->show();
         unsigned long state = NET::Sticky | NET::StaysOnTop | NET::KeepAbove;
         KWindowSystem::setState(m_controlBar->effectiveWinId(), state);
         KWindowSystem::setType(m_controlBar->effectiveWinId(), NET::Dock);
+
+        if (m_shadowWindow && m_shadowWindow->isValid()) {
+            Plasma::WindowEffects::slideWindow(m_shadowWindow, m_controlBar->location());
+            m_shadowWindow->show();
+            if (!m_autoHideControlBar) {
+                KWindowSystem::setState(m_shadowWindow->winId(), NET::KeepBelow);
+            }
+            KWindowSystem::setOnAllDesktops(m_shadowWindow->winId(), true);
+        }
     } else if (!m_autoHideControlBar) {
         Plasma::WindowEffects::slideWindow(m_controlBar, m_controlBar->location());
         m_controlBar->hide();
         createUnhideTrigger();
+
+        if (m_shadowWindow) {
+            Plasma::WindowEffects::slideWindow(m_shadowWindow, m_controlBar->location());
+            m_shadowWindow->hide();
+        }
     }
 }
 
@@ -987,7 +1203,7 @@ void PlasmaApp::unhideHintMousePoll()
         m_glowBar = 0;
         XMoveResizeWindow(QX11Info::display(), m_unhideTrigger, m_unhideTriggerGeom.x(), m_unhideTriggerGeom.y(), m_unhideTriggerGeom.width(), m_unhideTriggerGeom.height());
     } else {
-        controlBarVisibilityUpdate();
+        m_unHideTimer->start(0);
     }
 #endif
 }
@@ -1015,22 +1231,24 @@ void PlasmaApp::createUnhideTrigger()
 
     switch (m_controlBar->location()) {
         case Plasma::TopEdge:
-            actualWidth = triggerWidth = m_controlBar->width();
+            actualWidth = triggerWidth = m_controlBar->width() - 1;
             actualHeight = 1;
+            triggerPoint += QPoint(1, 0);
 
             break;
         case Plasma::BottomEdge:
-            actualWidth = triggerWidth = m_controlBar->width();
-            actualTriggerPoint = triggerPoint = m_controlBar->geometry().bottomLeft();
+            actualWidth = triggerWidth = m_controlBar->width() - 1;
+            actualTriggerPoint = triggerPoint = m_controlBar->geometry().bottomLeft() + QPoint(1, 0);
 
             break;
         case Plasma::RightEdge:
-            actualHeight = triggerHeight = m_controlBar->height();
-            actualTriggerPoint = triggerPoint = m_controlBar->geometry().topRight();
+            actualHeight = triggerHeight = m_controlBar->height() - 1;
+            actualTriggerPoint = triggerPoint = m_controlBar->geometry().topRight() + QPoint(0, 1);
 
             break;
         case Plasma::LeftEdge:
-            actualHeight = triggerHeight = m_controlBar->height();
+            actualHeight = triggerHeight = m_controlBar->height() - 1;
+            triggerPoint += QPoint(0, -1);
 
             break;
         default:
