@@ -21,18 +21,15 @@
 
 #include <QApplication>
 #include <QStandardItemModel>
-#include <QVBoxLayout>
-#include <QHBoxLayout>
-#include <QSpinBox>
-#include <QBitArray>
 #include <QBitmap>
+#include <QBitArray>
 
-#include <KAction>
 #include <KIconLoader>
 #include <KIconTheme>
 #include <KMenu>
 #include <KStandardAction>
 #include <KStringHandler>
+#include <KAction>
 
 #ifdef FOUND_SOPRANO
 #include <Soprano/Node>
@@ -131,17 +128,17 @@ void EngineExplorer::dataUpdated(const QString& source, const Plasma::DataEngine
 {
     QList<QStandardItem*> items = m_dataModel->findItems(source, 0);
 
-    if (items.count() < 1) {
+    if (items.isEmpty()) {
         return;
     }
 
     QStandardItem* parent = items.first();
 
-    while (parent->hasChildren()) {
-        parent->removeRow(0);
-    }
+    int rows = showData(parent, data);
 
-    showData(parent, data);
+    while (parent->rowCount() > rows) {
+        parent->removeRow(rows);
+    }
 }
 
 void EngineExplorer::listEngines()
@@ -184,11 +181,11 @@ void EngineExplorer::showEngine(const QString& name)
         return;
     }
 
-    QStringList sources = m_engine->sources();
-
     //kDebug() << "showing engine " << m_engine->objectName();
     //kDebug() << "we have " << sources.count() << " data sources";
-    foreach (const QString& source, sources) {
+    connect(m_engine, SIGNAL(sourceAdded(QString)), this, SLOT(addSource(QString)));
+    connect(m_engine, SIGNAL(sourceRemoved(QString)), this, SLOT(removeSource(QString)));
+    foreach (const QString& source, m_engine->sources()) {
         //kDebug() << "adding " << source;
         addSource(source);
     }
@@ -197,19 +194,20 @@ void EngineExplorer::showEngine(const QString& name)
     m_updateInterval->setEnabled(true);
     m_sourceRequester->setEnabled(true);
     m_sourceRequester->setFocus();
-    connect(m_engine, SIGNAL(sourceAdded(QString)), this, SLOT(addSource(QString)));
-    connect(m_engine, SIGNAL(sourceRemoved(QString)), this, SLOT(removeSource(QString)));
     updateTitle();
 }
 
 void EngineExplorer::addSource(const QString& source)
 {
+    QList<QStandardItem*> items = m_dataModel->findItems(source, 0);
+    if (!items.isEmpty()) {
+        return;
+    }
+
     QStandardItem* parent = new QStandardItem(source);
     m_dataModel->appendRow(parent);
 
     //kDebug() << "getting data for source " << source;
-    Plasma::DataEngine::Data data = m_engine->query(source);
-    showData(parent, data);
 
     if (!m_requestingSource || m_sourceRequester->text() != source) {
         m_engine->connectSource(source, this);
@@ -268,7 +266,7 @@ void EngineExplorer::showDataContextMenu(const QPoint &point)
             index = m_dataModel->index(index.row(), 0);
         }
 
-        QString source = index.data().toString();
+        const QString source = index.data().toString();
         KMenu menu;
         menu.addTitle(source);
         QAction *service = menu.addAction(i18n("Get associated service"));
@@ -324,7 +322,16 @@ QString EngineExplorer::convertToString(const QVariant &value) const
             return QString("%1").arg(value.toLocale().name());
         }
         case QVariant::Map: {
-            return i18np("&lt;1 item&gt;", "&lt;%1 items&gt;", value.toMap().size());
+            QVariantMap map = value.toMap();
+            QString str = i18np("&lt;1 item&gt;", "&lt;%1 items&gt;", map.size());
+
+            QMapIterator<QString, QVariant> it(map);
+            while (it.hasNext()) {
+                it.next();
+                str += "\n" + it.key() + ": " + convertToString(it.value());
+            }
+
+            return str;
         }
         case QVariant::Pixmap: {
             QPixmap pixmap = value.value<QPixmap>();
@@ -402,7 +409,7 @@ QString EngineExplorer::convertToString(const QVariant &value) const
     }
 }
 
-void EngineExplorer::showData(QStandardItem* parent, Plasma::DataEngine::Data data)
+int EngineExplorer::showData(QStandardItem* parent, Plasma::DataEngine::Data data)
 {
     int rowCount = 0;
     Plasma::DataEngine::DataIterator it(data);
@@ -419,15 +426,22 @@ void EngineExplorer::showData(QStandardItem* parent, Plasma::DataEngine::Data da
                 parent->setChild(rowCount, 3, new QStandardItem(var.typeName()));
                 ++rowCount;
             }
-        }
-        else {
-            QStandardItem *item = new QStandardItem(convertToString(it.value()));
+        } else {
+            QStandardItem *item;
+            if (it.value().canConvert<QIcon>()) {
+                item = new QStandardItem(it.value().value<QIcon>(), "");
+            } else {
+                item = new QStandardItem(convertToString(it.value()));
+            }
+
             item->setToolTip(item->text());
             parent->setChild(rowCount, 2, item);
             parent->setChild(rowCount, 3, new QStandardItem(it.value().typeName()));
             ++rowCount;
         }
     }
+
+    return rowCount;
 }
 
 void EngineExplorer::updateTitle()
@@ -442,6 +456,7 @@ void EngineExplorer::updateTitle()
                              "%1 Engine - 1 data source", "%1 Engine - %2 data sources")
                               .subs(KStringHandler::capwords(m_engine->name()))
                               .subs(m_sourceCount).toString());
+
     if (m_engine->icon().isEmpty()) {
         m_title->setPixmap(KIcon("plasma").pixmap(IconSize(KIconLoader::Dialog)));
     } else {
