@@ -41,7 +41,7 @@ NMNetworkManagerPrivate::NMNetworkManagerPrivate() : iface(NMNetworkManager::DBU
     kDebug(1441) << NMNetworkManager::DBUS_SERVICE;
 }
 
-NMNetworkManager::NMNetworkManager(QObject * parent, const QStringList &) 
+NMNetworkManager::NMNetworkManager(QObject * parent, const QVariantList &) 
 {
     qDBusRegisterMetaType<QList<QDBusObjectPath> >();
     d_ptr = new NMNetworkManagerPrivate;
@@ -68,7 +68,7 @@ NMNetworkManager::NMNetworkManager(QObject * parent, const QStringList &)
     {
         kDebug(1441) << "Device list";
         QList <QDBusObjectPath> devices = deviceList.value();
-        foreach (QDBusObjectPath op, devices)
+        foreach (const QDBusObjectPath &op, devices)
         {
             d->networkInterfaces.append(op.path());
             kDebug(1441) << "  " << op.path();
@@ -79,7 +79,7 @@ NMNetworkManager::NMNetworkManager(QObject * parent, const QStringList &)
 
     kDebug(1441) << "Active connections:";
     QList <QDBusObjectPath> activeConnections = d->iface.activeConnections();
-    foreach (QDBusObjectPath ac, activeConnections)
+    foreach (const QDBusObjectPath &ac, activeConnections)
     {
         d->activeConnections.append(ac.path());
         kDebug(1441) << "  " << ac.path();
@@ -161,7 +161,7 @@ void NMNetworkManager::activateConnection(const QString & interfaceUni, const QS
     // ### FIXME find a better name for the parameter needed for NM 0.7
     QString extra_connection_parameter = connectionParameters.value("extra_connection_parameter").toString();
     if (extra_connection_parameter.isEmpty()) {
-        extra_connection_parameter = QLatin1String("/foo");
+        extra_connection_parameter = QLatin1String("/");
     }
     if ( serviceName.isEmpty() || connectionPath.isEmpty() ) {
         return;
@@ -230,7 +230,7 @@ void NMNetworkManager::propertiesChanged(const QVariantMap &properties)
         if ( activePaths.count() ) {
             kDebug(1441) << activeConnKey;
         }
-        foreach (QDBusObjectPath ac, activePaths)
+        foreach (const QDBusObjectPath &ac, activePaths)
         {
             d->activeConnections.append(ac.path());
             kDebug(1441) << "  " << ac.path();
@@ -292,5 +292,16 @@ QStringList NMNetworkManager::activeConnections() const
     return d->activeConnections;
 }
 
+Solid::Control::NetworkInterface::Types NMNetworkManager::supportedInterfaceTypes() const
+{
+    return (Solid::Control::NetworkInterface::Types) (
+           Solid::Control::NetworkInterface::Ieee80211 |
+           Solid::Control::NetworkInterface::Ieee8023 |
+           Solid::Control::NetworkInterface::Gsm |
+           Solid::Control::NetworkInterface::Cdma |
+           Solid::Control::NetworkInterface::Serial
+           );
+}
+
 #include "manager.moc"
 
