@@ -40,7 +40,7 @@ public:
     static const QString DBUS_USER_SETTINGS_PATH;
     static const QString DBUS_SYSTEM_SETTINGS_PATH;
 
-    NMNetworkManager(QObject * parent, const QStringList & args);
+    NMNetworkManager(QObject * parent, const QVariantList & args);
     ~NMNetworkManager();
     Solid::Networking::Status status() const;
     QStringList networkInterfaces() const;
@@ -57,12 +57,7 @@ public Q_SLOTS:
     void setWirelessEnabled(bool enabled);
 
 Q_SIGNALS:
-    void statusChanged(Solid::Networking::Status status);
-    void networkInterfaceAdded(const QString & uni);
-    void networkInterfaceRemoved(const QString & uni);
-    void wirelessEnabledChanged(bool);
     void networkingEnabledChanged(bool);
-    void activeConnectionsChanged();
 
 protected Q_SLOTS:
     void deviceAdded(const QDBusObjectPath &state);
